Package: nipalsMCIA
Title: Multiple Co-Inertia Analysis via the NIPALS Method
Version: 1.11.0
Authors@R: c(
    person("Maximilian", "Mattessich",
            email = "maximilian.mattessich@northwestern.edu",
            role = "cre", comment = c(ORCID = "0000-0002-1233-1240")),
    person("Joaquin", "Reyna", email = "joreyna@live.com",
            role = "aut", comment = c(ORCID = "0000-0002-8468-2840")),
    person("Edel", "Aron", email = "edel.aron@yale.edu",
            role = "aut", comment = c(ORCID = "0000-0002-8683-4772")),
    person("Ferhat", "Ay", email = "ferhatay@lji.org",
            role = "aut", comment = c(ORCID = "0000-0002-0708-6914")),
    person("Steven", "Kleinstein", email = "steven.kleinstein@yale.edu",
            role = "aut", comment = c(ORCID = "0000-0003-4957-1544")),
    person("Anna", "Konstorum", email = "konstorum.anna@gmail.com",
            role = "aut", comment = c(ORCID = "0000-0003-4024-2058")))
Description: Computes Multiple Co-Inertia Analysis (MCIA), a
        dimensionality reduction (jDR) algorithm, for a multi-block
        dataset using a modification to the Nonlinear Iterative Partial
        Least Squares method (NIPALS) proposed in (Hanafi et. al,
        2010). Allows multiple options for row- and table-level
        preprocessing, and speeds up computation of variance explained.
        Vignettes detail application to bulk- and single cell-
        multi-omics studies.
License: GPL-3
URL: https://github.com/Muunraker/nipalsMCIA
BugReports: https://github.com/Muunraker/nipalsMCIA/issues
Depends: R (>= 4.3.0)
Imports: ComplexHeatmap, dplyr, fgsea, ggplot2 (>= 3.0.0), graphics,
        grid, methods, MultiAssayExperiment, SummarizedExperiment,
        pracma, rlang, RSpectra, scales, stats
Suggests: BiocFileCache, BiocStyle, circlize, ggpubr, KernSmooth,
        knitr, piggyback, reshape2, rmarkdown, rpart, Seurat (>=
        4.0.0), spatstat.explore, stringr, survival, tidyverse,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
biocViews: Software, Clustering, Classification, MultipleComparison,
        Normalization, Preprocessing, SingleCell
Encoding: UTF-8
RoxygenNote: 7.3.2
Config/testthat/edition: 3
Config/pak/sysreqs: libicu-dev libpng-dev perl zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 13:01:33 UTC
RemoteUrl: https://github.com/bioc/nipalsMCIA
RemoteRef: HEAD
RemoteSha: e4c24cee0df32d41e2b3cca1c4f283b169d4a939
NeedsCompilation: no
Packaged: 2026-05-30 07:30:16 UTC; root
Author: Maximilian Mattessich [cre] (ORCID:
    <https://orcid.org/0000-0002-1233-1240>),
  Joaquin Reyna [aut] (ORCID: <https://orcid.org/0000-0002-8468-2840>),
  Edel Aron [aut] (ORCID: <https://orcid.org/0000-0002-8683-4772>),
  Ferhat Ay [aut] (ORCID: <https://orcid.org/0000-0002-0708-6914>),
  Steven Kleinstein [aut] (ORCID:
    <https://orcid.org/0000-0003-4957-1544>),
  Anna Konstorum [aut] (ORCID: <https://orcid.org/0000-0003-4024-2058>)
Maintainer: Maximilian Mattessich <maximilian.mattessich@northwestern.edu>
Built: R 4.6.0; ; 2026-05-30 10:45:21 UTC; unix

# Generated by roxygen2: do not edit by hand

export(block_preproc)
export(block_weights_heatmap)
export(cc_preproc)
export(col_preproc)
export(deflate_block_bl)
export(deflate_block_gs)
export(extract_from_mae)
export(get_colors)
export(get_metadata_colors)
export(get_tv)
export(global_scores_eigenvalues_plot)
export(global_scores_heatmap)
export(gsea_report)
export(nipals_iter)
export(nipals_multiblock)
export(nmb_get_bl)
export(nmb_get_bs)
export(nmb_get_bs_weights)
export(nmb_get_eigs)
export(nmb_get_gl)
export(nmb_get_gs)
export(nmb_get_metadata)
export(ord_loadings)
export(predict_gs)
export(projection_plot)
export(simple_mae)
export(vis_load_ord)
export(vis_load_plot)
exportClasses(NipalsResult)
importClassesFrom(MultiAssayExperiment,MultiAssayExperiment)
importFrom(MultiAssayExperiment,assays)
importFrom(MultiAssayExperiment,colData)
importFrom(MultiAssayExperiment,experiments)
importFrom(MultiAssayExperiment,intersectColumns)
importFrom(MultiAssayExperiment,mergeReplicates)
importFrom(RSpectra,svds)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(graphics,barplot)
importFrom(graphics,grid)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(methods,is)
importFrom(methods,new)
importFrom(pracma,rand)
importFrom(rlang,quo)
importFrom(scales,viridis_pal)
importFrom(stats,cov)
importFrom(stats,setNames)
importFrom(stats,var)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# nipalsMCIA: Software to Compute Multi-Block Dimensionality Reduction

<!-- badges: start -->

[![BioC
status](http://www.bioconductor.org/shields/build/release/bioc/nipalsMCIA.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/nipalsMCIA)
[![R-CMD-check](https://github.com/Muunraker/nipalsMCIA/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Muunraker/nipalsMCIA/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

This package computes Multiple Co-Inertia Analysis (MCIA) on multi-block
data using the Nonlinear Iterative Partial Least Squares (NIPALS)
method.

Features include:

- Efficient computation of deflation and variance enabling embedding of
  high-volume (e.g. single-cell) datasets.  
- Functionality to perform out-of-sample embedding.
- Easy-to-adjust options for deflation and pre-processing
- Multiple visualization and analysis options for sample- and
  feature-level embedding results
- Streamlined and well-documented and supported code that is consistent
  with published theory to enable more efficient algorithm development
  and extension

**Citation**

For more information on the methodology used in nipalsMCIA and to cite,
please see

- Maximilian Mattessich, Joaquin Reyna, Edel Aron, Ferhat Ay, Misha Kilmer, Steven H Kleinstein, Anna Konstorum, nipalsMCIA: flexible multi-block dimensionality reduction in R via nonlinear iterative partial least squares, Bioinformatics, Volume 41, Issue 1, January 2025, btaf015, https://doi.org/10.1093/bioinformatics/btaf015

## Installation

This package can be installed [via
Bioconductor](https://bioconductor.org/packages/release/bioc/html/nipalsMCIA.html):

``` r
if (!require("BiocManager", quietly = TRUE))
  install.packages("BiocManager")

BiocManager::install("nipalsMCIA")
```

You can install the development version of nipalsMCIA from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("Muunraker/nipalsMCIA", ref = "devel",
                         force = TRUE, build_vignettes = TRUE)
```

## Basic Example

The package currently includes one test dataset: `data_blocks`. This is
a list of dataframes containing observations of variables from three
omics types (mRNA, proteins, and micro RNA) on 21 cancer cell lines from
the NCI60 cancer cell lines. The data file includes a `metadata` data
frame containing the cancer type associated with each cell line.

``` r
# load the package and set a seed for reproducibility
library(nipalsMCIA)
set.seed(42)
```

``` r
data(NCI60) # import data as "data_blocks" and metadata as "metadata_NCI60"

# examine the data and metadata
summary(data_blocks)
#>       Length Class      Mode
#> mrna  12895  data.frame list
#> miRNA   537  data.frame list
#> prot   7016  data.frame list
head(metadata_NCI60)
#>            cancerType
#> CNS.SF_268        CNS
#> CNS.SF_295        CNS
#> CNS.SF_539        CNS
#> CNS.SNB_19        CNS
#> CNS.SNB_75        CNS
#> CNS.U251          CNS
table(metadata_NCI60)
#> cancerType
#>      CNS Leukemia Melanoma 
#>        6        6        9
```

*Note: this dataset is reproduced from the [omicade4
package](https://www.bioconductor.org/packages/release/bioc/html/omicade4.html)
(Meng et. al., 2014). This package assumes all input datasets are in
sample by feature format.*

The main MCIA function can be called on `data_blocks` and optionally can
include `metadata_NCI60` for plot coloring by cancer type:

``` r
# to convert data_blocks into an MAE object we provide the simple_mae() function
data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
                              colData = metadata_NCI60)

mcia_results <- nipals_multiblock(data_blocks_mae = data_blocks_mae,
                                  col_preproc_method = "colprofile",
                                  num_PCs = 10, tol = 1e-12,
                                  color_col = "cancerType")
```

<img src="man/figures/README-call-mcia-1.png" width="100%" style="display: block; margin: auto;" />

Here `num_PCs` is the dimension of the low-dimensional embedding of the
data chosen by the user.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----installation-github, eval = FALSE----------------------------------------
# # devel version
# 
# # install.packages("devtools")
# devtools::install_github("Muunraker/nipalsMCIA", ref = "devel",
#                          force = TRUE, build_vignettes = TRUE) # devel version

## ----installation-bioconductor, eval = FALSE----------------------------------
# # release version
# if (!require("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# 
# BiocManager::install("nipalsMCIA")

## ----load-packages, message = FALSE-------------------------------------------
library(ggplot2)
library(MultiAssayExperiment)
library(nipalsMCIA)

## ----split-data---------------------------------------------------------------
data(NCI60)
set.seed(8)

num_samples <- dim(data_blocks[[1]])[1]
num_train <- round(num_samples * 0.7, 0)
train_samples <- sample.int(num_samples, num_train)

data_blocks_train <- data_blocks
data_blocks_test <- data_blocks

for (i in seq_along(data_blocks)) {
  data_blocks_train[[i]] <- data_blocks_train[[i]][train_samples, ]
  data_blocks_test[[i]] <- data_blocks_test[[i]][-train_samples, ]
}

# Split corresponding metadata
metadata_train <- data.frame(metadata_NCI60[train_samples, ],
                             row.names = rownames(data_blocks_train$mrna))
colnames(metadata_train) <- c("cancerType")

metadata_test <- data.frame(metadata_NCI60[-train_samples, ],
                            row.names = rownames(data_blocks_test$mrna))
colnames(metadata_test) <- c("cancerType")

# Create train and test mae objects
data_blocks_train_mae <- simple_mae(data_blocks_train, row_format = "sample",
                                    colData = metadata_train)
data_blocks_test_mae <- simple_mae(data_blocks_test, row_format = "sample",
                                   colData = metadata_test)

## ----computing-MCIA-on-training-data, message = FALSE, fig.show = "hide"------
MCIA_train <- nipals_multiblock(data_blocks = data_blocks_train_mae,
                                col_preproc_method = "colprofile", num_PCs = 10,
                                plots = "none", tol = 1e-9)

## ----visualize-training-model, fig.height = 2.5-------------------------------
meta_colors <- get_metadata_colors(mcia_results = MCIA_train, color_col = 1,
                                   color_pal_params = list(option = "E"))

global_scores <- nmb_get_gs(MCIA_train)
MCIA_out <- data.frame(global_scores[, 1:2])
MCIA_out$cancerType <- nmb_get_metadata(MCIA_train)$cancerType
colnames(MCIA_out) <- c("Factor.1", "Factor.2", "cancerType")

# plot the results
ggplot(data = MCIA_out, aes(x = Factor.1, y = Factor.2, color = cancerType)) +
  geom_point(size = 3) +
  labs(title = "MCIA for NCI60 training data") +
  scale_color_manual(values = meta_colors) +
  theme_bw()

## ----generate-new-scores, message = FALSE-------------------------------------
MCIA_test_scores <- predict_gs(mcia_results = MCIA_train,
                               test_data = data_blocks_test_mae)

## ----visualize-both-models, fig.height = 2.5----------------------------------
MCIA_out_test <- data.frame(MCIA_test_scores[, 1:2])
MCIA_out_test$cancerType <-
  MultiAssayExperiment::colData(data_blocks_test_mae)$cancerType

colnames(MCIA_out_test) <- c("Factor.1", "Factor.2", "cancerType")
MCIA_out_test$set <- "test"
MCIA_out$set <- "train"
MCIA_out_full <- rbind(MCIA_out, MCIA_out_test)
rownames(MCIA_out_full) <- NULL

# plot the results
ggplot(data = MCIA_out_full,
       aes(x = Factor.1, y = Factor.2, color = cancerType, shape = set)) +
  geom_point(size = 3) +
  labs(title = "MCIA for NCI60 training and test data") +
  scale_color_manual(values = meta_colors) +
  theme_bw()

## ----session-info-------------------------------------------------------------
sessionInfo()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----installation-github, eval = FALSE----------------------------------------
# # devel version:
# # install.packages("devtools")
# devtools::install_github("Muunraker/nipalsMCIA", ref = "devel",
#                          force = TRUE, build_vignettes = TRUE) # devel version

## ----installation-bioconductor, eval = FALSE----------------------------------
# # release version:
# if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")
# 
# BiocManager::install("nipalsMCIA")

## ----load-packages, message = FALSE-------------------------------------------
# note that the TENxPBMCData package is not included in this list as you may
# decide to pull data from another source or use our provided objects
packages <- c("BiocFileCache", "dplyr", "ggplot2", "ggpubr", "nipalsMCIA",
              "piggyback", "Seurat")

# load each package and print the version
for (n in seq_along(packages)) {
  suppressPackageStartupMessages(library(packages[n], character.only = TRUE))
  cat(paste0(packages[n], ": ", packageVersion(packages[n]), "\n"))
}

# note that as of Seurat v5, GetAssayData() uses `layer` instead of `slot`

# NIPALS starts with a random vector
set.seed(42)

## ----set-paths----------------------------------------------------------------
# if you would like to save any of the data loaded/created locally
path_data <- file.path("..", "data")

## ----cache--------------------------------------------------------------------
# as suggested in: https://bioconductor.org/packages/release/bioc/vignettes/BiocFileCache/inst/doc/BiocFileCache.html#cache-to-manage-package-data
# since we only use the cache in this vignette, we don't need wrapper functions

# initialize the cache in a temporary directory
bfc <- BiocFileCache::BiocFileCache()

# you can run cleanbfc(bfc, days = -Inf) if you want to remove everything
  
# set up or load the unique resource ids
rid <- BiocFileCache::bfcquery(bfc, query = "vignette_sc", "rname")$rid
# rid <- bfcrid(bfc) # we only have one rname, so you could also just use this

# if the data is not in the cache, download and add it
if (length(rid) == 0) {
  # specify `tag = ` to use a different release other than latest
  urls <-
    piggyback::pb_download_url(repo = "Muunraker/nipalsMCIA", tag = "latest")
  
  # you could add all of the urls to rid at once, but it would throw this error
  # when it comes to downloading them:
  # "Error in parse_url(url) : length(url) == 1 is not TRUE"
  for (url in urls) {
    # the rids are numbered in the order they are added e.g. BFC1, BFC2, ...
    message(paste("Downloading file", url))
    rid <- names(BiocFileCache::bfcadd(bfc, rname = "vignette_sc", url))
    
    # check if the cached data needs updating and download it if needed
    if (!isFALSE(BiocFileCache::bfcneedsupdate(bfc, rid))) {
      # this will overwrite existing files without asking
      BiocFileCache::bfcdownload(bfc, rid, ask = FALSE)
    }
  }
}

# return the paths to the cached files
# BiocFileCache::bfcrpath(bfc, rid = rid)

## ----cache-temp-dir-----------------------------------------------------------
# you can also run tools::R_user_dir("BiocFileCache", which = "cache")
BiocFileCache::getBFCOption("CACHE")

## ----pipelines, echo = FALSE--------------------------------------------------
# pipeline image
knitr::include_graphics(path = BiocFileCache::bfcquery(bfc, "Pipeline")$rpath)

## ----data-all-list------------------------------------------------------------
# list all of the currently available files in the latest release
piggyback::pb_list(repo = "Muunraker/nipalsMCIA", tag = "latest")

## ----data-all-download--------------------------------------------------------
# files needed for running MCIA
path_metadata_sc <-
  BiocFileCache::bfcquery(bfc, query = "metadata_sc")$rpath

# MCIA results
# path_data_blocks <- bfcquery(bfc, query = "data_blocks")$rpath
path_mcia_results <-
  BiocFileCache::bfcquery(bfc, query = "mcia_results_sc")$rpath

# marker genes for cell type annotation with Seurat
path_marker_genes <-
  BiocFileCache::bfcquery(bfc, query = "marker_genes")$rpath

# the Seurat data's metric summary file from 10x Genomics
path_metrics_summary <-
  BiocFileCache::bfcquery(bfc, query = "metrics_summary")$rpath

# Seurat objects in different stages of processing
path_obj_raw <-
  BiocFileCache::bfcquery(bfc, query = "CITEseq_raw")$rpath
path_obj_annotated <-
  BiocFileCache::bfcquery(bfc, query = "CITEseq_annotated")$rpath

## ----data-bioconductor-load, eval = FALSE-------------------------------------
# # read in the data as a SingleCellExperiment object
# tenx_pbmc5k <- TENxPBMCData::TENxPBMCData(dataset = "pbmc5k-CITEseq")
# 
# # examine the data
# tenx_pbmc5k
# ## class: SingleCellExperiment
# ## dim: 33538 5247
# ## metadata(0):
# ## assays(1): counts
# ## rownames(33538): ENSG00000243485 ENSG00000237613 ... ENSG00000277475 ENSG00000268674
# ## rowData names(4): ENSEMBL_ID Symbol_TENx Type Symbol
# ## colnames: NULL
# ## colData names(11): Sample Barcode ... Individual Date_published
# ## reducedDimNames(0):
# ## mainExpName: Gene Expression
# ## altExpNames(1): Antibody Capture
# 
# counts(tenx_pbmc5k)
# ## <33538 x 5247> sparse matrix of class DelayedMatrix and type "integer":
# ##                    [, 1]    [, 2]    [, 3]    [, 4] ... [, 5244] [, 5245] [, 5246] [, 5247]
# ## ENSG00000243485       0       0       0       0   .       0       0       0       0
# ## ENSG00000237613       0       0       0       0   .       0       0       0       0
# ## ENSG00000186092       0       0       0       0   .       0       0       0       0
# ## ENSG00000238009       0       0       0       0   .       0       0       0       0
# ## ENSG00000239945       0       0       0       0   .       0       0       0       0
# ##             ...       .       .       .       .   .       .       .       .       .
# ## ENSG00000277856       0       0       0       0   .       0       0       0       0
# ## ENSG00000275063       0       0       0       0   .       0       0       0       0
# ## ENSG00000271254       0       0       0       0   .       0       0       0       0
# ## ENSG00000277475       0       0       0       0   .       0       0       0       0
# ## ENSG00000268674       0       0       0       0   .       0       0       0       0
# 
# counts(altExp(tenx_pbmc5k))
# ## <32 x 5247> sparse matrix of class DelayedMatrix and type "integer":
# ##           [, 1]    [, 2]    [, 3]    [, 4] ... [, 5244] [, 5245] [, 5246] [, 5247]
# ##    CD3      25     959     942     802   .     402     401       6    1773
# ##    CD4     164     720    1647    1666   .    1417       1      46    1903
# ##   CD8a      16       8      21       5   .       8     222       3       9
# ##  CD11b    3011      12      11      11   .      15       7    1027       9
# ##   CD14     696      12      13       9   .       9      17     382       8
# ##    ...       .       .       .       .   .       .       .       .       .
# ## HLA-DR     573      15      11      19   .       6      40     184      32
# ##  TIGIT      10       3       3       3   .       2      15       1      12
# ##   IgG1       4       4       2       4   .       1       0       2       4
# ##  IgG2a       1       3       0       6   .       4       0       4       2
# ##  IgG2b       6       2       4       8   .       0       0       2       5
# 
# # examine the metadata:
# head(colData(tenx_pbmc5k), n = 3)
# ## DataFrame with 6 rows and 11 columns
# ##           Sample            Barcode         Sequence   Library Cell_ranger_version Tissue_status Barcode_type
# ##      <character>        <character>      <character> <integer>         <character>   <character>  <character>
# ## 1 pbmc5k-CITEseq AAACCCAAGAGACAAG-1 AAACCCAAGAGACAAG         1              v3.0.2            NA     Chromium
# ## 2 pbmc5k-CITEseq AAACCCAAGGCCTAGA-1 AAACCCAAGGCCTAGA         1              v3.0.2            NA     Chromium
# ## 3 pbmc5k-CITEseq AAACCCAGTCGTGCCA-1 AAACCCAGTCGTGCCA         1              v3.0.2            NA     Chromium
# ##     Chemistry Sequence_platform   Individual Date_published
# ##   <character>       <character>  <character>    <character>
# ## 1 Chromium_v3           NovaSeq HealthyDonor     2019-05-29
# ## 2 Chromium_v3           NovaSeq HealthyDonor     2019-05-29
# ## 3 Chromium_v3           NovaSeq HealthyDonor     2019-05-29
# 
# head(rowData(tenx_pbmc5k), n = 3)
# ## DataFrame with 6 rows and 4 columns
# ##                      ENSEMBL_ID Symbol_TENx            Type       Symbol
# ##                     <character> <character>     <character>  <character>
# ## ENSG00000243485 ENSG00000243485 MIR1302-2HG Gene Expression           NA
# ## ENSG00000237613 ENSG00000237613     FAM138A Gene Expression      FAM138A
# ## ENSG00000186092 ENSG00000186092       OR4F5 Gene Expression        OR4F5
# 
# metadata(tenx_pbmc5k)
# ## list()
# 
# # change the gene names from Ensembl IDs to the 10x genes
# rownames(tenx_pbmc5k) <- rowData(tenx_pbmc5k)$Symbol_TENx

## ----data-bioconductor-formatting, eval = FALSE-------------------------------
# # set up the list
# data_blocks_sc_sce <- list()
# data_blocks_sc_sce$mrna <- data.frame(as.matrix(counts(tenx_pbmc5k)))
# data_blocks_sc_sce$adt <- data.frame(as.matrix(counts(altExp(tenx_pbmc5k))))
# 
# summary(data_blocks_sc_sce)
# ##      Length Class      Mode
# ## mrna 5247   data.frame list
# ## adt  5247   data.frame list
# 
# # convert to a Seurat object (using `as.Seurat` won't work here)
# obj_sce <- CreateSeuratObject(counts = data_blocks_sc_sce$mrna, # assay = "RNA"
#                               project = "pbmc5k_CITEseq")
# obj_sce[["ADT"]] <- CreateAssayObject(counts = data_blocks_sc_sce$adt)
# 
# # name the cells with their barcodes
# obj_sce <- RenameCells(object = obj_sce,
#                        new.names = colData(tenx_pbmc5k)$Sequence)
# 
# # add metadata from the SingleCellExperiment object
# obj_sce <- AddMetaData(object = obj_sce,
#                        metadata = as.data.frame(colData(tenx_pbmc5k),
#                                                 row.names = Cells(obj_sce)))
# 
# # this object will be slightly different than from the Seurat one down below
# # e.g. 5297 rows vs. 4193 rows (since QC wasn't done) and different metadata
# 
# head(obj_sce[[]], n = 3)
# ##                     orig.ident nCount_RNA nFeature_RNA nCount_ADT nFeature_ADT         Sample            Barcode
# ## AAACCCAAGAGACAAG SeuratProject       7375         2363       5178           31 pbmc5k-CITEseq AAACCCAAGAGACAAG-1
# ## AAACCCAAGGCCTAGA SeuratProject       3772         1259       2893           29 pbmc5k-CITEseq AAACCCAAGGCCTAGA-1
# ## AAACCCAGTCGTGCCA SeuratProject       4902         1578       3635           29 pbmc5k-CITEseq AAACCCAGTCGTGCCA-1
# ##                          Sequence Library Cell_ranger_version Tissue_status Barcode_type   Chemistry Sequence_platform
# ## AAACCCAAGAGACAAG AAACCCAAGAGACAAG       1              v3.0.2          <NA>     Chromium Chromium_v3           NovaSeq
# ## AAACCCAAGGCCTAGA AAACCCAAGGCCTAGA       1              v3.0.2          <NA>     Chromium Chromium_v3           NovaSeq
# ## AAACCCAGTCGTGCCA AAACCCAGTCGTGCCA       1              v3.0.2          <NA>     Chromium Chromium_v3           NovaSeq
# ##                    Individual Date_published
# ## AAACCCAAGAGACAAG HealthyDonor     2019-05-29
# ## AAACCCAAGGCCTAGA HealthyDonor     2019-05-29
# ## AAACCCAGTCGTGCCA HealthyDonor     2019-05-29
# 
# # save the data locally if desired
# save(data_blocks_sc_sce, obj_sce,
#      file = file.path(path_data, "data_sc_sce.Rda"))

## ----data-10x-object, eval = FALSE--------------------------------------------
# # load the data (change the file path as needed)
# data <- Seurat::Read10X(data.dir = file.path(path_data, "tenx_pbmc5k_CITEseq",
#                                              "filtered_feature_bc_matrix"),
#                         strip.suffix = TRUE) # remove the "-1"s from barcodes
# ## 10X data contains more than one type and is being returned as a list
# ## containing matrices of each type.
# 
# # set minimum cells and/or features here if you'd like
# obj <- Seurat::CreateSeuratObject(counts = data$`Gene Expression`,
#                                   project = "pbmc5k_CITEseq")
# obj[["ADT"]] <- Seurat::CreateAssayObject(counts = data$`Antibody Capture`)
# ## Warning: Feature names cannot have underscores ('_'), replacing with dashes ('-')
# 
# # check the assays
# Seurat::Assays(object = obj)
# ## "RNA" "ADT"
# 
# # list out the CITE-Seq surface protein markers
# rownames(obj[["ADT"]])
# ## [1] "CD3-TotalSeqB"            "CD4-TotalSeqB"           "CD8a-TotalSeqB"
# ## [4] "CD11b-TotalSeqB"          "CD14-TotalSeqB"          "CD15-TotalSeqB"
# ## [7] "CD16-TotalSeqB"           "CD19-TotalSeqB"          "CD20-TotalSeqB"
# ## [10] "CD25-TotalSeqB"          "CD27-TotalSeqB"          "CD28-TotalSeqB"
# ## [13] "CD34-TotalSeqB"          "CD45RA-TotalSeqB"        "CD45RO-TotalSeqB"
# ## [16] "CD56-TotalSeqB"          "CD62L-TotalSeqB"         "CD69-TotalSeqB"
# ## [19] "CD80-TotalSeqB"          "CD86-TotalSeqB"          "CD127-TotalSeqB"
# ## [22] "CD137-TotalSeqB"         "CD197-TotalSeqB"         "CD274-TotalSeqB"
# ## [25] "CD278-TotalSeqB"         "CD335-TotalSeqB"         "PD-1-TotalSeqB"
# ## [28] "HLA-DR-TotalSeqB"        "TIGIT-TotalSeqB"         "IgG1-control-TotalSeqB"
# ## [31] "IgG2a-control-TotalSeqB" "IgG2b-control-TotalSeqB"
# 
# # save the data locally if desired
# saveRDS(obj, file = file.path(path_data, "tenx_pbmc5k_CITEseq_raw.rds"))

## ----mcia-metadata------------------------------------------------------------
# read in the annotated cells
metadata_sc <- read.csv(file = path_metadata_sc[1], header = TRUE, row.names = 1)

# examples
metadata_sc %>% slice_sample(n = 5)

## ----mcia-decomp-load-data, eval = FALSE--------------------------------------
# # load the object set up for running MCIA [10x Genomics & Seurat]
# # (if you save it in the following code block or download it from a release)
# load(file = path_data_blocks)

## ----mcia-decomp-run, eval = FALSE--------------------------------------------
# # "largest_sv" results in a more balanced contribution
# # from the blocks than the default "unit_var"
# set.seed(42)
# 
# # convert data_blocks_sc to an MAE object using the SingleCellExperiment class
# data_blocks_sc_mae <-
#   MultiAssayExperiment::MultiAssayExperiment(lapply(data_blocks_sc, function(x)
#     SingleCellExperiment::SingleCellExperiment(t(as.matrix(x)))),
#     colData = metadata_sc)
# 
# mcia_results_sc <- nipals_multiblock(data_blocks = data_blocks_sc_mae,
#                                      col_preproc_method = "colprofile",
#                                      block_preproc_method = "largest_sv",
#                                      num_PCs = 10, tol = 1e-9,
#                                      deflationMethod = "global",
#                                      plots = "none")
# ## Performing column-level pre-processing...
# ## Column pre-processing completed.
# ## Performing block-level preprocessing...
# ## Block pre-processing completed.
# ## Computing order 1 scores
# ## Computing order 2 scores
# ## Computing order 3 scores
# ## Computing order 4 scores
# ## Computing order 5 scores
# ## Computing order 6 scores
# ## Computing order 7 scores
# ## Computing order 8 scores
# ## Computing order 9 scores
# ## Computing order 10 scores
# 
# # saveRDS(mcia_results_sc, file = file.path(path_data, "mcia_results_sc.Rds"))

## ----mcia-decomp-load---------------------------------------------------------
# load the results of the previous block (if already run and saved)
mcia_results_sc <- readRDS(file = path_mcia_results[1])
mcia_results_sc

## ----mcia-plots-colors--------------------------------------------------------
# for the projection plot
# technically you could just do color_pal_params = list(option = "D"), but saving
# the colors is useful for other plots like in the Seurat section
meta_colors_sc <- get_metadata_colors(mcia_results = mcia_results_sc,
                                      color_col = "CellType",
                                      color_pal = scales::viridis_pal,
                                      color_pal_params = list(option = "D"))

# for other plots
colors_omics_sc <- get_colors(mcia_results = mcia_results_sc)

## ----mcia-plots-eigenvalue-scree, fig.dim = c(5, 4)---------------------------
global_scores_eigenvalues_plot(mcia_results = mcia_results_sc)

## ----mcia-plots-projection, fig.dim = c(5, 5)---------------------------------
projection_plot(mcia_results = mcia_results_sc,
                projection = "global", orders = c(1, 2),
                color_col = "CellType", color_pal = meta_colors_sc,
                legend_loc = "bottomright")

## ----mcia-plots-heatmap-global, fig.dim = c(7, 5)-----------------------------
suppressMessages(global_scores_heatmap(mcia_results = mcia_results_sc,
                                       color_col = "CellType",
                                       color_pal = meta_colors_sc))

## ----mcia-plots-heatmap-block, fig.dim = c(4, 2.5)----------------------------
block_weights_heatmap(mcia_results = mcia_results_sc)

## ----mcia-plots-loadings, fig.dim = c(6, 4.5)---------------------------------
vis_load_plot(mcia_results = mcia_results_sc,
              axes = c(1, 4), color_pal = colors_omics_sc)

## ----mcia-plots-top-features-factor-1, fig.dim = c(10, 4)---------------------
all_pos_1_vis <- vis_load_ord(mcia_results_sc, omic = "all", factor = 1, 
                              absolute = FALSE, descending = TRUE)
mrna_pos_1_vis <- vis_load_ord(mcia_results_sc, omic = "mrna", factor = 1, 
                               absolute = FALSE, descending = TRUE)

ggpubr::ggarrange(all_pos_1_vis, mrna_pos_1_vis, widths = c(1, 1))

## ----mcia-plots-top-features-factor-1-orderings-------------------------------
all_pos_1 <- ord_loadings(mcia_results_sc, omic = "all", factor = 1, 
                              absolute = FALSE, descending = TRUE)
mrna_pos_1 <- ord_loadings(mcia_results_sc, omic = "mrna", factor = 1, 
                               absolute = FALSE, descending = TRUE)

## ----mcia-plots-top-features-factor-1-orderings-1-----------------------------
# visualize the tabular data
all_pos_1[1:3, ]

## ----mcia-plots-top-features-factor-1-orderings-2-----------------------------
mrna_pos_1[1:3, ]

## ----mcia-plots-top-features-factor-4, fig.dim = c(10, 5)---------------------
vis_load_ord(mcia_results_sc, omic = "all", factor = 4, n_feat = 60,
             absolute = FALSE, descending = TRUE)

## ----seurat-load-data, message = FALSE----------------------------------------
# load the data
obj <- readRDS(file = path_obj_raw[1])

# add useful metadata
obj[["percent.mt"]] <- PercentageFeatureSet(object = obj, pattern = "^MT-")

## ----seurat-qc-metrics-summary------------------------------------------------
# read in and display the summary table
metrics_summary <- read.csv(file = path_metrics_summary[1])
metrics_summary

## ----seurat-qc-pre-filter, fig.dim = c(8, 5)----------------------------------
VlnPlot(object = obj,
        features = c("nFeature_RNA", "nCount_RNA", "percent.mt"),
        pt.size = 0.01, ncol = 3) &
  theme(axis.text.x = element_text(angle = 0, hjust = 0.5),
        axis.title.x = element_blank())

## ----seurat-qc-post-filter, fig.dim = c(8, 5)---------------------------------
# adjust cutoffs as desired
obj <- subset(x = obj, subset = nFeature_RNA > 200 & percent.mt < 20)

VlnPlot(object = obj,
        features = c("nFeature_RNA", "nCount_RNA", "percent.mt"),
        pt.size = 0.01, ncol = 3) &
  theme(axis.text.x = element_text(angle = 0, hjust = 0.5),
        axis.title.x = element_blank())

## ----seurat-pipeline-1, eval = FALSE------------------------------------------
# # standard log normalization for RNA and centered log for ADT
# obj <- NormalizeData(object = obj, normalization.method = "LogNormalize",
#                      scale.factor = 10000, assay = "RNA")
# ## Performing log-normalization
# ## 0%   10   20   30   40   50   60   70   80   90   100%
# ## [----|----|----|----|----|----|----|----|----|----|
# ## **************************************************|
# 
# obj <- NormalizeData(object = obj, normalization.method = "CLR",
#                      margin = 2, assay = "ADT") # go across cells, not features
# ## Normalizing across cells
# ##   |++++++++++++++++++++++++++++++++++++++++++++++++++| 100% elapsed=00s
# 
# # highly variable features
# obj <- FindVariableFeatures(object = obj, selection.method = "vst",
#                             nfeatures = 2000)
# ## Calculating gene variances
# ## 0%   10   20   30   40   50   60   70   80   90   100%
# ## [----|----|----|----|----|----|----|----|----|----|
# ## **************************************************|
# ## Calculating feature variances of standardized and clipped values
# ## 0%   10   20   30   40   50   60   70   80   90   100%
# ## [----|----|----|----|----|----|----|----|----|----|
# ## **************************************************|
# 
# # scaling so the average expression is 0 and the variance is 1
# obj <- ScaleData(object = obj, features = rownames(obj))
# ## Centering and scaling data matrix
# ##   |===============================================================================================| 100%
# 
# # dimensionality reduction
# obj <- RunPCA(object = obj)
# ## PC_ 1
# ## Positive:  LYZ, FCN1, CST3, MNDA, CTSS, PSAP, S100A9, FGL2, AIF1, GRN
# ##     NCF2, LST1, CD68, TYMP, SERPINA1, CYBB, CLEC12A, CSTA, SPI1, TNFAIP2
# ##     CPVL, VCAN, MPEG1, TYROBP, KLF4, FTL, S100A8, IGSF6, CD14, MS4A6A
# ## Negative:  CD3D, TRAC, LTB, TRBC2, IL32, CD3G, IL7R, CD69, CD247, TRBC1
# ##     CD2, CD7, CD27, ARL4C, ISG20, HIST1H4C, SYNE2, GZMM, ITM2A, CCR7
# ##     RORA, MAL, CXCR4, LEF1, TRAT1, CTSW, GZMA, KLRB1, TRABD2A, CCL5
# ## PC_ 2
# ## Positive:  CD79A, MS4A1, IGHM, BANK1, HLA-DQA1, CD79B, IGKC, LINC00926, RALGPS2, TNFRSF13C
# ##     VPREB3, IGHD, SPIB, CD22, FCRL1, HLA-DQB1, BLK, FAM129C, FCRLA, TCL1A
# ##     GNG7, TCF4, COBLL1, PAX5, SWAP70, CD40, BCL11A, P2RX5, TSPAN13, ADAM28
# ## Negative:  NKG7, CST7, GZMA, PRF1, KLRD1, CTSW, FGFBP2, GNLY, GZMH, CCL5
# ##     GZMM, CD247, KLRF1, HOPX, SPON2, ADGRG1, TRDC, MATK, GZMB, FCGR3A
# ##     S100A4, CCL4, CLIC3, KLRB1, IL2RB, TBX21, TTC38, ANXA1, PTGDR, PLEKHF1
# ## PC_ 3
# ## Positive:  GZMB, NKG7, GNLY, CLIC3, PRF1, KLRD1, FGFBP2, KLRF1, SPON2, CST7
# ##     GZMH, FCGR3A, ADGRG1, GZMA, HOPX, CTSW, TRDC, CCL4, HLA-DPB1, C12orf75
# ##     PLAC8, TTC38, PLEK, APOBEC3G, TBX21, PRSS23, CYBA, MATK, SYNGR1, CXXC5
# ## Negative:  IL7R, MAL, LEF1, TRABD2A, TRAC, CCR7, LTB, CD27, FOS, LRRN3
# ##     FHIT, TRAT1, RGCC, CAMK4, CD3D, RGS10, CD40LG, FOSB, AQP3, SOCS3
# ##     FLT3LG, CD3G, SLC2A3, TSHZ2, VIM, S100A12, S100A8, CD28, PLK3, VCAN
# ## PC_ 4
# ## Positive:  FCER1A, PLD4, SERPINF1, IL3RA, CLEC10A, GAS6, LILRA4, TPM2, CLEC4C, ENHO
# ##     FLT3, SMPD3, ITM2C, LGMN, CD1C, P2RY14, PPP1R14B, SCT, PROC, LAMP5
# ##     RUNX2, AC119428.2, PACSIN1, DNASE1L3, PTCRA, RGS10, UGCG, CLIC2, PPM1J, P2RY6
# ## Negative:  MS4A1, CD79A, LINC00926, BANK1, TNFRSF13C, VPREB3, CD79B, RALGPS2, IGHD, FCRL1
# ##     BLK, IGHM, CD22, PAX5, ARHGAP24, CD24, P2RX5, NCF1, S100A12, CD19
# ##     SWAP70, FCRLA, VNN2, TNFRSF13B, FCER2, IGKC, FCRL2, RBP7, CD40, S100A8
# ## PC_ 5
# ## Positive:  BATF3, C1QA, TCF7L2, CTSL, CDKN1C, HLA-DQB1, HES4, SIGLEC10, CLEC10A, ABI3
# ##     HLA-DQA1, RHOC, CSF1R, ENHO, CAMK1, MTSS1, IFITM3, CD1C, LY6E, FCGR3A
# ##     HLA-DPA1, CLIC2, HLA-DPB1, YBX1, RRAS, AC064805.1, NR4A1, GBP1, ZNF703, CXCL16
# ## Negative:  LILRA4, TPM2, CLEC4C, SMPD3, IL3RA, SERPINF1, DERL3, MZB1, SCT, JCHAIN
# ##     PACSIN1, PROC, S100A12, PTCRA, LINC00996, PADI4, ASIP, KCNK17, ITM2C, EPHB1
# ##     ALOX5AP, LAMP5, DNASE1L3, MAP1A, S100A8, APP, CYP1B1, VNN2, UGCG, ZFAT
# 
# # clustering (adjust dimensions and resolutions as desired)
# obj <- FindNeighbors(object = obj, reduction = "pca", dims = 1:20)
# ## Computing nearest neighbor graph
# ## Computing SNN
# 
# obj <- FindClusters(object = obj, resolution = 0.4)
# ## Modularity Optimizer version 1.3.0 by Ludo Waltman and Nees Jan van Eck
# ##
# ## Number of nodes: 4193
# ## Number of edges: 154360
# ##
# ## Running Louvain algorithm...
# ## 0%   10   20   30   40   50   60   70   80   90   100%
# ## [----|----|----|----|----|----|----|----|----|----|
# ## **************************************************|
# ## Maximum modularity in 10 random starts: 0.9097
# ## Number of communities: 10
# ## Elapsed time: 0 seconds
# 
# obj <- RunUMAP(object = obj, reduction = "pca", dims = 1:20)
# ## 14:44:35 UMAP embedding parameters a = 0.9922 b = 1.112
# ## 14:44:35 Read 4193 rows and found 20 numeric columns
# ## 14:44:35 Using Annoy for neighbor search, n_neighbors = 30
# ## 14:44:35 Building Annoy index with metric = cosine, n_trees = 50
# ## 0%   10   20   30   40   50   60   70   80   90   100%
# ## [----|----|----|----|----|----|----|----|----|----|
# ## **************************************************|
# ## 14:44:36 Writing NN index file to temp file /tmp/RtmpHhoQAU/filec346711dd9f93
# ## 14:44:36 Searching Annoy index using 1 thread, search_k = 3000
# ## 14:44:37 Annoy recall = 100%
# ## 14:44:38 Commencing smooth kNN distance calibration using 1 thread with target n_neighbors = 30
# ## 14:44:39 Initializing from normalized Laplacian + noise (using RSpectra)
# ## 14:44:39 Commencing optimization for 500 epochs, with 174318 positive edges
# ## Using method 'umap'
# ## 0%   10   20   30   40   50   60   70   80   90   100%
# ## [----|----|----|----|----|----|----|----|----|----|
# ## **************************************************|
# ## 14:44:44 Optimization finished

## ----seurat-load-processed-object---------------------------------------------
obj <- readRDS(file = path_obj_annotated[1])

# reset to baseline
Idents(object = obj) <- "seurat_clusters"
obj$annotated_clusters <- c()

## ----seurat-elbow, fig.dim = c(6, 3)------------------------------------------
ElbowPlot(object = obj, ndims = 30)

## ----seurat-top-features------------------------------------------------------
head(VariableFeatures(object = obj), 20)

## ----seurat-top-features-plot, fig.dim = c(6, 6), warning = FALSE, eval = FALSE----
# # these can be plotted if desired
# LabelPoints(plot = VariableFeaturePlot(object = obj),
#             points = head(VariableFeatures(object = obj), 20),
#             repel = TRUE, xnudge = 0, ynudge = 0) +
#   labs(title = "Top Twenty Variable Features") +
#   theme(legend.position = "bottom")

## ----seurat-umap-initial, fig.dim = c(6, 6), message = FALSE------------------
# you can also use the LabelClusters function to help label individual clusters
plot_seurat_clusters <-
  UMAPPlot(object = obj, label = TRUE, label.size = 4, label.box = TRUE) +
  labs(title = "Initial Clusters") +
  scale_fill_manual(values = rep("white", n_distinct(obj$seurat_clusters))) +
  theme(plot.title = element_text(hjust = 0.5),
        axis.text = element_blank(), axis.ticks = element_blank())

plot_seurat_clusters

## ----seurat-markers-load------------------------------------------------------
# sort by Cell_Type and Marker if not already sorted
markers_all <- read.csv(file = path_marker_genes[1])
markers_all %>% slice_sample(n = 10) # example markers

## ----seurat-markers-dot-gex, fig.dim = c(16, 6), warning = FALSE--------------
select_cell_types <- c("B", "Macrophages", "mDC", "NK", "T")

# do features = unique(markers_all$Marker) to use all possible features
p <- DotPlot(object = obj,
             features = markers_all %>%
                          dplyr::filter(Cell_Type %in% select_cell_types) %>%
                          distinct(Marker) %>% pull(),
             cols = "RdBu", col.min = -1, dot.scale = 3, cluster.idents = TRUE)

# add in the cell type information
# if desired, you could rename the "Cell_Type"s in the original database to be
# more informative e.g. Natural_Killers instead of NK
p$data <- left_join(p$data,
                    markers_all %>%
                      dplyr::filter(Cell_Type %in% select_cell_types) %>%
                      dplyr::rename(features.plot = "Marker"),
                    by = "features.plot", multiple = "all")
# depending on your version of dplyr, you can set `relationship = "many-to-many"`
# to surpress the warning

# plot
p +
  facet_grid(cols = vars(Cell_Type), scales = "free_x", space = "free") +
  theme(strip.text.x = element_text(size = 10)) + RotatedAxis()

## ----seurat-markers-dot-adt, fig.dim = c(16, 6)-------------------------------
# you have to change the default assay for the dot plot
DefaultAssay(object = obj) <- "ADT"

DotPlot(object = obj,
        features =
          rownames(GetAssayData(object = obj, layer = "counts", assay = "ADT")),
        cols = "RdBu", col.min = -1, dot.scale = 3, cluster.idents = TRUE) +
  RotatedAxis()

# reset the default
DefaultAssay(object = obj) <- "RNA"

## ----seurat-markers-feature, fig.dim = c(12, 8)-------------------------------
FeaturePlot(object = obj,
            features = c("MS4A1", "CD14", # MS4A1 is another name for CD20
                         "NKG7", "IL3RA", "CD3D", "IL7R"),
            min.cutoff = 0, label = TRUE, label.size = 4,
            ncol = 3, raster = FALSE)

## ----seurat-markers-violin-adt, fig.dim = c(8, 8)-----------------------------
VlnPlot(object = obj,
        features = c("CD3-TotalSeqB", "CD14-TotalSeqB", "CD20-TotalSeqB",
                     "CD335-TotalSeqB"),
        pt.size = 0, assay = "ADT", ncol = 2) &
  theme(plot.title = element_text(size = 10),
        axis.text.x = element_text(angle = 0, hjust = 0.5),
        axis.title.x = element_blank())

## ----seurat-annotations-------------------------------------------------------
# mDC = myeloid dendritic cells
# pDC = plasmacytoid dendritic cells
obj_annotations <- rbind(c("0", "Macrophages/mDCs"), # difficult to separate
                         c("1", "T Cells"),
                         c("2", "T Cells"),
                         c("3", "T Cells"),
                         c("4", "Natural Killers"),
                         c("5", "B Cells"),
                         c("6", "Macrophages/mDCs"), # difficult to separate
                         c("7", "Macrophages/mDCs"), # difficult to separate
                         c("8", "T Cells"), # faint signal
                         c("9", "pDCs"))
colnames(obj_annotations) <- c("Cluster", "CellType")
obj_annotations <- data.frame(obj_annotations)

# save the annotations as a csv if you'd like
# write.csv(obj_annotations, file = file.path(path_data, "obj_annotations.csv"))

# prepare the annotation information
annotations <- setNames(obj_annotations$CellType, obj_annotations$Cluster)

# relabel the Seurat clusters
# Idents(obj) <- "seurat_clusters"
obj <- RenameIdents(object = obj, annotations)

# alphabetize the cell types
Idents(obj) <- factor(Idents(object = obj), levels = sort(levels(obj)))

# useful metadata (e.g. if you want to have multiple annotation sets)
obj[["annotated_clusters"]] <- Idents(object = obj)

# save the processed and annotated Seurat object
# saveRDS(obj, file = file.path(path_data, "tenx_pbmc5k_CITEseq_annotated.rds"))

# info about the clusters
obj_annotations %>%
  group_by(CellType) %>%
  transmute(Clusters = paste0(Cluster, collapse = ", ")) %>%
  distinct() %>%
  arrange(CellType)

# metadata file for MCIA
meta <- data.frame("CellType" = Idents(object = obj))

# save the metadata file for MCIA
# write.csv(meta, file = file.path(path_data, "metadata_sc.csv"))

## ----seurat-umap-both, fig.dim = c(16, 8), message = FALSE--------------------
# change colors and themes as desired
plot_annotated_clusters <-
  UMAPPlot(object = obj, cols = meta_colors_sc, label = TRUE, label.size = 4,
           label.box = TRUE) +
  labs(title = "Annotated Clusters") +
  scale_fill_manual(values = rep("white", length(meta_colors_sc))) +
  theme(plot.title = element_text(hjust = 0.5),
        axis.text = element_blank(), axis.ticks = element_blank()) +
  NoLegend() # a legend is not needed here

ggpubr::ggarrange(plot_seurat_clusters + NoLegend(), plot_annotated_clusters,
                  heights = c(1, 1), widths = c(1, 1))

## ----seurat-umap-adt, fig.dim = c(6, 6)---------------------------------------
# plot ADT information on top as an example
FeaturePlot(object = obj, features = "adt_CD19-TotalSeqB",
            label = TRUE, label.size = 4, ncol = 1, raster = FALSE)

## ----seurat-check-annotations-violin, fig.dim = c(8, 8)-----------------------
VlnPlot(object = obj,
        features = c("CD3D", "CD4", "IL7R", "TRAC"),
        cols = meta_colors_sc, pt.size = 0.01, ncol = 2) &
  theme(plot.title = element_text(size = 10),
        axis.title.x = element_blank())

## ----seurat-check-annotations-bar, fig.dim = c(8, 6)--------------------------
# examine the total counts
ggplot(obj[[]], aes(x = annotated_clusters, fill = annotated_clusters)) +
  geom_bar(color = "black", linewidth = 0.2) +
  labs(title = "pbmc5k-CITEseq Cell Type Counts",
       x = "Cell Type", y = "Count") +
  scale_fill_manual(values = meta_colors_sc) +
  theme_bw() +
  theme(plot.title = element_text(size = 14, hjust = 0.5),
        axis.text.x = element_text(angle = 45, hjust = 1),
        panel.grid.major.x = element_blank(), legend.position = "none")

## ----seurat-save-mcia, eval = FALSE-------------------------------------------
# # mrna
# data_rna <- GetAssayData(object = obj, layer = "data",
#                          assay = "RNA")[VariableFeatures(object = obj), ]
# data_rna <- as.matrix(data_rna)
# data_rna <- t(data_rna) # switch the rows
# # colnames(data_rna) <- paste(colnames(data_rna), "mrna", sep = "_")
# 
# # adt
# data_adt <- GetAssayData(object = obj, layer = "data", assay = "ADT")
# data_adt <- as.matrix(data_adt)
# data_adt <- t(data_adt) # switch the rows
# # colnames(data_adt) <- paste(colnames(data_adt), "adt", sep = "_")
# 
# # combined
# data_blocks_sc <- list(mrna = data_rna, adt = data_adt)
# 
# # examine the contents
# data.frame(data_blocks_sc$mrna[1:5, 1:5])
# data.frame(data_blocks_sc$adt[1:5, 1:5])
# 
# # save(data_blocks_sc, file = file.path(path_data, "data_blocks_sc.Rda"))

## ----session-info-------------------------------------------------------------
sessionInfo()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----installation-github, eval = FALSE----------------------------------------
# # devel version
# 
# # install.packages("devtools")
# devtools::install_github("Muunraker/nipalsMCIA", ref = "devel",
#                          force = TRUE, build_vignettes = TRUE) # devel version

## ----installation-bioconductor, eval = FALSE----------------------------------
# # release version
# if (!require("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# 
# BiocManager::install("nipalsMCIA")

## ----load-packages, message = FALSE-------------------------------------------
library(ComplexHeatmap)
library(dplyr)
library(fgsea)
library(ggplot2)
library(ggpubr)
library(nipalsMCIA)
library(stringr)

# NIPALS starts with a random vector
set.seed(42)

## ----intro-load-data----------------------------------------------------------
# load the dataset which uses the name data_blocks
data(NCI60)

# examine the contents
data_blocks$miRNA[1:3, 1:3]
data_blocks$mrna[1:3, 1:3]
data_blocks$prot[1:3, 1:3]

## ----convert-to-mae-----------------------------------------------------------
data_blocks_mae <- simple_mae(data_blocks, row_format = "sample")

## ----intro-mcia, warning = FALSE, message = FALSE-----------------------------
set.seed(42)
mcia_results <- nipals_multiblock(data_blocks_mae,
                                  col_preproc_method = "colprofile",
                                  num_PCs = 10, tol = 1e-12, plots = "none")

## ----intro-mcia-results-------------------------------------------------------
slotNames(mcia_results)

## ----part-1-mcia, warning = FALSE, message = FALSE, fig.dim = c(9, 5)---------
set.seed(42)
mcia_results <- nipals_multiblock(data_blocks_mae,
                                  col_preproc_method = "colprofile",
                                  num_PCs = 10, tol = 1e-12)

## ----part-1-visualize-clusters, fig.dim = c(5, 5)-----------------------------
projection_plot(mcia_results, projection = "global", orders = c(1, 2))

## ----part-1-metadata----------------------------------------------------------
# preview of metadata
head(metadata_NCI60)

# loading of mae with metadata
data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
                              colData = metadata_NCI60)

## ----part-1-mcia-again, warning = FALSE, message = FALSE----------------------
# adding metadata as part of the nipals_multiblock() function
set.seed(42)
mcia_results <- nipals_multiblock(data_blocks_mae,
                                  col_preproc_method = "colprofile",
                                  plots = "none",
                                  num_PCs = 10, tol = 1e-12)

## ----part-1-visualize-clusters-color-col, fig.dim = c(5, 5)-------------------
# meta_colors = c("blue", "grey", "yellow") can use color names
# meta_colors = c("#00204DFF", "#7C7B78FF", "#FFEA46FF") can use hex codes
meta_colors <- get_metadata_colors(mcia_results, color_col = 1,
                                   color_pal_params = list(option = "E"))

projection_plot(mcia_results, projection = "global", orders = c(1, 2),
                color_col = "cancerType", color_pal = meta_colors,
                legend_loc = "bottomleft")

## ----part-1-global-heatmap-colored, fig.dim = c(6, 4)-------------------------
global_scores_heatmap(mcia_results = mcia_results,
                      color_col = "cancerType", color_pal = meta_colors)

## ----part-2-block-heatmap, fig.dim = c(4.5, 3)--------------------------------
block_weights_heatmap(mcia_results)

## ----part-2-loadings, fig.dim = c(6, 4)---------------------------------------
# colors_omics = c("red", "blue", "green")
# colors_omics <- get_colors(mcia_results, color_pal = colors_omics)
colors_omics <- get_colors(mcia_results)

vis_load_plot(mcia_results, axes = c(1, 4), color_pal = colors_omics)

## ----part-2-factor-1, fig.dim = c(8, 3)---------------------------------------
# generate the visualizations
all_pos_1_vis <- vis_load_ord(mcia_results, omic = "all", factor = 1,
                              absolute = FALSE, descending = TRUE)
mrna_pos_1_vis <- vis_load_ord(mcia_results, omic = "mrna", factor = 1,
                               absolute = FALSE, descending = TRUE)

ggpubr::ggarrange(all_pos_1_vis, mrna_pos_1_vis)

## ----part-2-factor-1-orderings------------------------------------------------
# obtain the loadings as plotted above
all_pos_1 <- ord_loadings(mcia_results, omic = "all", factor = 1,
                          absolute = FALSE, descending = TRUE)
mrna_pos_1 <- ord_loadings(mcia_results, omic = "mrna", factor = 1,
                           absolute = FALSE, descending = TRUE)

## ----part-2-factor-1-orderings-1----------------------------------------------
# visualize the tabular data
all_pos_1[1:3, ]

## ----part-2-factor-1-orderings-2----------------------------------------------
mrna_pos_1[1:3, ]

## ----part-2-factor-2, fig.dim = c(8, 3)---------------------------------------
# define the loadings
all_abs_2 <- vis_load_ord(mcia_results, omic = "all", factor = 2,
                          absolute = TRUE, descending = TRUE)

prot_abs_2 <- vis_load_ord(mcia_results, omic = "prot", factor = 2,
                           absolute = TRUE, descending = TRUE)

ggpubr::ggarrange(all_abs_2, prot_abs_2)

## ----part-2-factor-2-orderings------------------------------------------------
# obtain the loadings as plotted above
all_abs_2_vis <- ord_loadings(mcia_results, omic = "all", factor = 2,
                          absolute = TRUE, descending = TRUE)
prot_abs_2_vis <- ord_loadings(mcia_results, omic = "prot", factor = 2,
                          absolute = TRUE, descending = TRUE)

## ----part-2-factor-2-orderings-1----------------------------------------------
# visualize the tabular data
all_abs_2_vis[1:3, ]

## ----part-2-factor-2-orderings-2----------------------------------------------
prot_abs_2_vis[1:3, ]

## ----part-2-factor-4, fig.dim = c(10, 4)--------------------------------------
# visualization
all_4_plot <- vis_load_ord(mcia_results, omic = "all", factor = 4, 
                           absolute = FALSE, descending = TRUE, n_feat = 60)
all_4_plot

## ----part-2-factor-4-orderings, fig.dim = c(10, 4)----------------------------
# visualize the tabular data
all_4 <- ord_loadings(mcia_results, omic = "all", factor = 4,
                      absolute = FALSE, descending = TRUE)

## ----part-2-pathways, echo = TRUE, message = FALSE, warning = FALSE, results = "hide"----
# extract mRNA global loadings
mrna_gfscores <- nmb_get_gl(mcia_results)
mrna_rows <- str_detect(row.names(mrna_gfscores), "_mrna")
mrna_gfscores <- mrna_gfscores[mrna_rows, ]

# rename rows to contain HUGO based gene symbols
row.names(mrna_gfscores) <- str_remove(rownames(mrna_gfscores), "_[0-9]*_.*")

# load pathway data
path.database <- "https://data.broadinstitute.org/gsea-msigdb/msigdb/release/6.2/c2.cp.reactome.v6.2.symbols.gmt"
pathways <- fgsea::gmtPathways(gmt.file = path.database)

# generate the GSEA report
geneset_report <- gsea_report(metagenes = mrna_gfscores, path.database,
                              factors = seq(1, 3), pval.thr = 0.05, nproc = 2)

## ----include = FALSE----------------------------------------------------------
# # Apply scientific notation to min_pval
geneset_report[[1]]$min_pval <- sprintf("%.2e", geneset_report[[1]]$min_pval)

## ----part-2-genesets----------------------------------------------------------
geneset_report[[1]]

## ----part-2-gsea-for-factor-3, echo = TRUE, message = FALSE, warning = FALSE, results = "hide"----
# re-running GSEA
factor3_paths <- fgseaMultilevel(pathways, stats = mrna_gfscores[, 3],
                                 nPermSimple = 10000, minSize = 15,
                                 nproc = 2, maxSize = 500)

## ----include=FALSE------------------------------------------------------------
# Order by adjusted p value
factor3_paths <- factor3_paths[order(factor3_paths$padj), ]

# Apply scientific notation to padj
to_sci <- function(x) return(sprintf("%.2e", x))
factor3_paths$padj <- vapply(factor3_paths$padj, to_sci, character(1))

# Clean up pathway name
clean_pathway_name <- function(x) {
    replaced_value <- str_replace_all(x , "_", " ")
    if (nchar(replaced_value) > 49) {
        substring_value <- substr(replaced_value, 1, 50) 
        return(paste0(substring_value, "..."))
    }
    else {
        return(replaced_value)
    }
}

factor3_paths[, "pathway"] <- apply(X = factor3_paths[, "pathway"],
                                    MARGIN = 1, FUN = clean_pathway_name)

## ----part-2-gsea-for-factor-3-viz---------------------------------------------
head(factor3_paths[, c("pathway", "padj")])

## ----part-2-significant-------------------------------------------------------
# extracting REACTOME_CELL_CYCLE, the most significant gene set
sig_path3 <- factor3_paths[min(factor3_paths$padj) == factor3_paths$padj, ][1, ]
sig_path3$leadingEdge[[1]][1:10]

## ----session-info-------------------------------------------------------------
sessionInfo()

#' Visualize ranked loadings
#'
#' @description Visualize a scree plot of loadings recovered from
#' nipalsMCIA() output loadings matrix ranked using the
#' ord_loadings() functions
#'
#' @param mcia_results object returned from nipals_multiblock() function
#' @param omic name of the given omic dataset
#' @param factor choose a factor (numeric value from 1 to number of factors
#'               in mcia_results)
#' @param n_feat number of features to visualize
#' @param absolute whether to rank by absolute value
#' @param descending whether to rank in descending or ascending order
#' @param color_pal a list of colors or function which returns a list of colors
#' @param color_pal_params a list of parameters for the color function
#' @return Plot in features for a factor by rank
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                   plots = "none", tol = 1e-12)
#' vis_load_ord(mcia_results, omic="mrna")
#' @importFrom rlang quo
#' @importFrom ggplot2 ggplot aes geom_point theme_bw xlab theme
#' @importFrom ggplot2 scale_color_manual element_text
#' @export
vis_load_ord <- function(mcia_results, omic, factor = 1, n_feat = 15,
                         absolute = TRUE, descending = TRUE,
                         color_pal = scales::viridis_pal,
                         color_pal_params = list()) {
    # get the colors
    plot_colors <- get_colors(mcia_results,
                              color_pal = color_pal,
                              color_pal_params = color_pal_params)

    # get the load ordering internally
    gl_f_ord <- ord_loadings(mcia_results = mcia_results, omic = omic,
                             factor = factor, absolute = absolute,
                             descending = descending)
    omic_name <- gl_f_ord$omic_name

    # setting parameters with tidy evaluation
    loading <- quo(`loading`)
    omic_quo <- quo(`omic`)

    # making a plot for the given omics
    n_plot <- min(nrow(gl_f_ord), n_feat)
    omic_subset <- names(table(droplevels(gl_f_ord[seq(0, n_plot), ])$omic))
    color_vals <- plot_colors[omic_subset]

    # plot the data
    ggplot(data = gl_f_ord[seq_len(n_plot), ],
           aes(x = factor(omic_name, levels = omic_name),
               y = !!loading, color = !!omic_quo)) +
        geom_point() +
        labs(x = "Feature", y = "Loading") + # color = "Omic"
        scale_color_manual(values = color_vals, limits = force) +
        theme_bw() +
        theme(axis.text.x = element_text(angle = 45, hjust = 1, size = 6))
}

mrna_pos_1 <- ord_loadings(mcia_results, omic = "mrna",
                           absolute = FALSE, descending = TRUE, factor = 1)
mrna_pos_1_vis <- vis_load_ord(mcia_results, omic = "mrna")

test_that("omic is correct", {
  expect_equal(as.character(unique(mrna_pos_1_vis$data$omic)[1]), "mrna")
})

test_that("factor is correct", {
  expect_equal(as.character(unique(mrna_pos_1_vis$data$factor)[1]), "1")
})

test_that("no errors produced", {
  expect_no_error(vis_load_ord(mcia_results, omic = "mrna"))
})

#' Plotting a heatmap of global factors scores (sample v. factors)
#'
#' @description Plots a heatmap of MCIA global scores
#' @param mcia_results the mcia object matrix after running MCIA, must also
#'     contain metadata with columns corresponding to color_col
#' @param color_col an integer or string specifying the column that will be
#'     used for color_col
#' @param color_pal a list of colors or function which returns a list of colors
#' @param color_pal_params a list of parameters for the color function
#' @return ComplexHeatmap object
#' @export
global_scores_heatmap <- function(mcia_results, color_col = NULL,
                                  color_pal = scales::viridis_pal,
                                  color_pal_params = list(option = "D")) {
    # extract the global scores
    global_scores <- mcia_results@global_scores

    # extract and re-label global scores
    colnames(global_scores) <- seq(1, ncol(global_scores))

    if (!is.null(color_col)) {
        # check if metadata is within the MCIA results
        if (nrow(mcia_results@metadata) == 0) {
          msg <- paste("Cannot produce a sample color_col without metadata.",
                       "Please re-run global_scores_heatmap after setting",
                       "this information.")
          stop(msg)
        }

        # extract the column names (if not provided via color_col)
        if (typeof(color_col) == "double") {
            color_col <- names(mcia_results@metadata)[color_col]
        }

        # extract color type and create a color palette list
        cat_values <- mcia_results@metadata[, color_col]

        cat_colors <- get_metadata_colors(mcia_results,
                                          color_col = color_col,
                                          color_pal = color_pal,
                                          color_pal_params = color_pal_params)
        cat_colors_list <- list(ColorType = cat_colors)

        # add the colors list to a HeatmapAnnotation obj
        row_ha <-
            ComplexHeatmap::HeatmapAnnotation(ColorType = cat_values,
                                              col = cat_colors_list,
                                              which = "row",
                                              show_annotation_name = FALSE,
                                              annotation_label = color_col)

        ComplexHeatmap::Heatmap(matrix = global_scores,
                                name = "Global Score",
                                row_title = "Samples",
                                column_title = "Factors",
                                row_names_side = "right",
                                show_row_names = TRUE,
                                row_names_gp = grid::gpar(fontsize = 7),
                                column_names_rot = 0,
                                column_names_centered = TRUE,
                                show_column_names = TRUE,
                                right_annotation = row_ha)
    } else {
        ComplexHeatmap::Heatmap(matrix = global_scores,
                                name = "Global Score",
                                row_title = "Samples",
                                column_title = "Factors",
                                row_names_side = "right",
                                show_row_names = TRUE,
                                row_names_gp = grid::gpar(fontsize = 7),
                                column_names_rot = 0,
                                column_names_centered = TRUE,
                                show_column_names = TRUE)
    }
}

test_that("checking figure", {
    expect_no_error(global_scores_heatmap(mcia_results,
                                          color_col = "cancerType")
    )
})

test_that("checking figure with custom color palette", {
    expect_no_error(global_scores_heatmap(mcia_results,
                                          color_col = "cancerType",
                                          color_pal = meta_colors)
    )
})

test_that("color column not in metadata", {
    expect_error(global_scores_heatmap(mcia_results_no_meta,
                                       color_col = "faux-column")
    )
})

test_that("checking the ggplot object with inputs", {
    # capture the plotting function
    p <- global_scores_heatmap(mcia_results, color_col = "cancerType")
    expect_equal(matrix(p@matrix), matrix(mcia_results@global_scores))
    expect_equal("cancerType", p@right_annotation@anno_list$ColorType@label)
})

gs_heatmap <- global_scores_heatmap(mcia_results)
test_that("Name is correct", {
    expect_equal(gs_heatmap@name, "Global Score")
})

#' Block-level preprocessing
#'
#' @description A function that normalizes an input dataset (data block)
#' according to a variety of options.
#' Intended to be used after column/row-level normalization.
#' @param df dataset to preprocess (must be in data matrix form)
#' @param block_preproc_method method which is used to normalize blocks, with
#' options: \itemize{
#' \item `unit_var` FOR CENTERED MATRICES ONLY - divides each block by the
#' square root of its variance
#' \item `num_cols` divides each block by the number of variables in the block.
#' \item `largest_sv` divides each block by its largest singular value.
#' \item `none` performs no preprocessing
#' }
#' @return the preprocessed dataset
#' @examples
#' df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
#' preprocessed_dataframe <- block_preproc(df,"unit_var")
#' @importFrom RSpectra svds
#' @export

block_preproc <- function(df, block_preproc_method) {
    temp_df <- df
    block_preproc_method <- tolower(block_preproc_method)

    # Weighting block to unit variance (block variance computed via Fro norm)
    if (block_preproc_method == "unit_var") {
        block_var <- norm(temp_df, type = "F")^2 / (max(1, nrow(temp_df) - 1))

        if (block_var != 0) {
            temp_df <- temp_df * (1 / sqrt(block_var))
        } else {
            warning("Data block has zero variance.")
        }

        return(temp_df)
    } else if (block_preproc_method == "num_cols") {
        temp_df <- temp_df * (1 / ncol(temp_df))
        return(temp_df)

    } else if (block_preproc_method == "largest_sv") {
        svdres <- RSpectra::svds(temp_df, 1)
        eigval <- svdres$d # largest singular value of the data block

        if (eigval != 0) {
            temp_df <- temp_df * (1 / eigval)
        } else {
            warning("Data block has zero largest eigenvalue")
        }

        return(temp_df)

    } else if (block_preproc_method == "none") {
        return(temp_df)

    } else {
        stop("block preprocessing method not recognized",
             "- pick from available options")
    }
}

test_that("block preprocessing by number of columns", {
    prot_num_cols <- prot / ncol(prot)

    expect_equal(block_preproc(prot, "num_cols"), prot_num_cols)
})


test_that("block preprocessing by unit variance", {
    block_var <- norm(as.matrix(prot), type = "F")^2 / (max(1, nrow(prot) - 1))
    prot_unit_var <- as.matrix(prot) * (1 / sqrt(block_var))

    expect_equal(block_preproc(as.matrix(prot), "unit_var"), prot_unit_var)
})

test_that("block preprocessing by largest_sv", {
    svdres <- RSpectra::svds(as.matrix(prot), 1)
    prot_svd <- as.matrix(prot) * (1 / svdres$d)

    expect_equal(block_preproc(as.matrix(prot), "largest_sv"), prot_svd)
})

test_that("block preprocessing by none", {
    expect_equal(block_preproc(prot, "none"), prot)
})

#' Ranked global loadings dataframe
#'
#' @description Creates a dataframe with ranked loadings for a given factor
#'
#' @param mcia_results object returned from nipals_multiblock() function
#' @param omic choose an omic to rank, or choose 'all' for all,
#' ((omic = "all", omic = "miRNA", etc.))
#' @param factor choose a factor (numeric value from 1 to number of factors
#'               in mcia_results)
#' @param absolute whether to rank by absolute value
#' @param descending whether to rank in descending or ascending order
#' @return ranked dataframe
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                   plots = "none", tol = 1e-12)
#' all_pos_1 <- ord_loadings(mcia_results = mcia_results, omic = "all",
#'                           absolute = FALSE, descending = TRUE, factor = 1)
#' @export
ord_loadings <- function(mcia_results, omic = "all", factor = 1,
                         absolute = FALSE, descending = TRUE) {
    # list of omics plus 'all'
    omics_names <- names(mcia_results@block_loadings)
    omics_names <- c(omics_names, "all")

    # Return error if omic not chosen from omics_names list
    if (!(omic %in% omics_names)) {
        stop("Choose an appropriate omic")
    }

    # Get global loadings and list of omics
    gl <- mcia_results@global_loadings
    # omic_type <- gsub("^.*_", "", rownames(gl))

    omic_dims <- vapply(mcia_results@block_loadings, dim, numeric(2))[1, ]
    omic_type <- c()
    omics_labels <- names(mcia_results@block_loadings)
    for (i in seq_along(omics_labels)) {
        omic_label <- omics_labels[i]
        length_omic <- omic_dims[i]
        omic_type <- c(omic_type, rep(omic_label, each = length_omic))
    }

    # Filter on factor, and add omic type
    gl_f <- data.frame(gl[, factor])
    gl_f$omic <- omic_type
    gl_f$omic <- as.factor(gl_f$omic)
    colnames(gl_f) <- c("loading", "omic")

    # Rank features by parameter settings
    if (absolute == FALSE) {
        if (descending == TRUE) {
            gl_f_ord <- gl_f[order(gl_f$loading, decreasing = TRUE), ]
        } else {
            gl_f_ord <- gl_f[order(gl_f$loading, decreasing = FALSE), ]
        }
    } else {
        gl_f_abs <- gl_f
        gl_f_abs$abs <- abs(gl_f_abs$loading)
        gl_f_ord <- gl_f_abs[order(gl_f_abs$abs, decreasing = TRUE), ]
    }

    if (omic != "all") {
        gl_f_ord <- gl_f_ord[gl_f_ord$omic == omic, ]
    }

    gl_f_ord$omic_name <- rownames(gl_f_ord)
    gl_f_ord$factor <- factor

    return(gl_f_ord)
}

test_that("only existing omics", {
    expect_error(ord_loadings(mcia_results, omic = "test"))
})

# all omics and factor 1, ranked in descending order
all_pos_1 <- ord_loadings(mcia_results, omic = "all",
                          absolute = FALSE, descending = TRUE, factor = 1)

test_that("factor filtration", {
    expect_equal(unique(all_pos_1$factor), 1)
})

test_that("all omics being used", {
    expect_equal(sort(as.character(unique(all_pos_1$omic))), # this is a factor
                 c("miRNA", "mrna", "prot"))
})

#' projection_plot
#'
#' @description Function to generate a projection plot of MCIA results.
#'
#' @details Plotting function for a projection plot.
#'
#' @param mcia_results MCIA results object returned from `nipals_multiblock`
#' @param projection of plot, with the following options
#'         \itemize{
#'         \item `all` - scatter plot of two orders of global and block
#'                       scores (aka factors).
#'         \item `global` - scatter plot of two orders of global scores
#'                          only (aka factors).
#'         \item `block` - scatter plot of two orders of block scores
#'                         only (aka factors) for given block.
#'         }
#' @param block_name Name of the block to be plotted (if `projection = block` argument used).
#'
#' @param orders Option to select orders of factors to plot against each other
#'         (for projection plots)
#' @param color_col Option to plot clusters/colors in projection plots,
#'         with two options:\itemize{
#' \item `none` (Default) - no clusters/colors plotted.
#' \item A character string of the column name of the `mcia_results@metadata`
#'         dataframe determining which color groupings to use
#'         projection plots only)
#' }
#' @param color_col an integer or string specifying the column that will be
#'     used for color_col
#' @param color_pal a list of colors or function which returns a list of colors
#' @param color_pal_params a list of parameters for the color function
#' @param legend_loc Option for legend location, or "none" for no legend.
#' @param color_override Option to override colors when necessary, helpful for
#' projection = "global" or "block"
#' @param cex Resizing parameter for drawing the points
#'
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                   plots = "none", tol = 1e-12)
#' projection_plot(mcia_results, projection = "all", orders = c(1,2),
#'                 color_col = "cancerType", legend_loc = "bottomright")
#' @return Displays the desired plots
#' @importFrom graphics grid legend points segments
#' @importFrom stats var
#' @export
projection_plot <- function(mcia_results, projection, orders = c(1, 2),
                            block_name = NULL, color_col = NULL,
                            color_pal = scales::viridis_pal,
                            color_pal_params = list(option = "E"),
                            legend_loc = "bottomleft", color_override = NULL,
                            cex = 0.5) {
    ### Identifying the membership of samples within
    ### the clusters/categories of the color_col column

    # case i) no color_col, clusters/categories were not specified
    if (is.null(color_col)) {
        clust_indexes <- list(seq(1, dim(mcia_results@global_scores)[[1]]))

    # case ii) yes color_col, clusters/categories were specified
    } else if (is.character(color_col)) {
        # locating the color_col column within metadata
        col_idx <- grep(color_col, names(mcia_results@metadata))
        if (any(length(col_idx) < 1)) {
            stop("Column name for color_col not found in metadata.")
        }

        # catching if two columns happen to have the same name.
        if (length(col_idx) > 1) {
            msg <- paste0("Metadata has duplicate columns for ", color_col,
                          ". Selecting the first one for plotting.")
            warning(msg)
        }
        col_idx <- col_idx[[1]]

        # creating a list of samples indexes to color each cluster/category
        clust_indexes <- list()
        uniq_clusts <- unique(unlist(mcia_results@metadata[col_idx]))
        for (clust in uniq_clusts) {
            cdata <- list(grep(clust, mcia_results@metadata[[col_idx]]))
            clust_indexes <- c(clust_indexes, cdata)
        }
        names(clust_indexes) <- uniq_clusts

    # case iii) catch-all, color_col argument not recognized
    } else {
        message("color_col option not recognized, ",
                "defaulting to black/white plotting.")
        clust_indexes <- list(seq(1, dim(mcia_results@global_scores)[[1]]))
    }

    ### Resolving the cluster colors IF SPECIFIED
    if (is.null(color_col)) {
        plot_colors <- list("black")
    } else {
    # only look up color if color_col is specified
        plot_colors <- get_metadata_colors(mcia_results, color_col = color_col,
                                           color_pal = color_pal,
                                           color_pal_params = color_pal_params)
    }

    if (!is.null(color_override)) {
        plot_colors <- list(color_override)
    }

    ### Plot 1 - projection == all
    if (tolower(projection) == "all") {
        # Normalize global scores to unit variance
        gs_norms <- apply(mcia_results@global_scores, 2,
                                            function(x) (sqrt(var(x))))
        gs_normed <- t(t(mcia_results@global_scores) / gs_norms)

        # Normalize block scores to unit variance
        bs_normed <- list()
        bl_normed <- list()
        for (i in seq(1, length(mcia_results@block_scores))) {
            bs_norms <- apply(mcia_results@block_scores[[i]], 2,
                                        function(x) (sqrt(var(x))))
            bs_normed[[i]] <- t(t(mcia_results@block_scores[[i]]) / bs_norms)
            bl_normed[[i]] <- t(t(mcia_results@block_loadings[[i]]) / bs_norms)
        }

        # Getting bounds for projection plot
        # minimum 1st block score
        min_bs1 <- min(vapply(lapply(bs_normed, `[`, , orders[[1]]),
                              min, numeric(1)))
        # minimum 2nd block score
        min_bs2 <- min(vapply(lapply(bs_normed, `[`, , orders[[2]]),
                              min, numeric(1)))
        # maximum 1st block score
        max_bs1 <- max(vapply(lapply(bs_normed, `[`, , orders[[1]]),
                              max, numeric(1)))
        # maximum 2nd block score
        max_bs2 <- max(vapply(lapply(bs_normed, `[`, , orders[[2]]),
                              max, numeric(1)))

        # minimum x coordinate in plot
        min_x <- min(c(min_bs1, min(gs_normed[, orders[[1]]])))
        # minimum y coordinate in plot
        min_y <- min(c(min_bs2, min(gs_normed[, orders[[2]]])))
        # maximum x coordinate in plot
        max_x <- max(c(max_bs1, max(gs_normed[, orders[[1]]])))
        # maximum y coordinate in plot
        max_y <- max(c(max_bs2, max(gs_normed[, orders[[2]]])))

        # Cluster 1
        sample_indexes <- clust_indexes[[1]]

        # catching error for one color
        if (is.null(color_col)) {
          plotCol <- plot_colors[[1]]
        } else {
          plotCol <- plot_colors[[names(clust_indexes)[1]]]
        }

        # Plotting global scores
        plot(gs_normed[sample_indexes, orders[[1]]],
             gs_normed[sample_indexes, orders[[2]]],
             main = "Factor Plot",
             xlab = paste("Factor", orders[[1]]),
             ylab = paste("Factor", orders[[2]]),
             col = plotCol,
             xlim = c(min_x, max_x),
             ylim = c(min_y, max_y),
             pch = 16, cex = cex)
        grid()

        # Plotting block scores (shapes correspond to different blocks)
        for (j in seq(1, length(bs_normed))) {
            bs_j <- bs_normed[[j]]
            points(bs_j[sample_indexes, orders[[1]]],
                   bs_j[sample_indexes, orders[[2]]],
                   col = plotCol, pch = j - 1, cex = 1)

            # Line segments joining block scores to central global score:
            segments(bs_j[sample_indexes, orders[[1]]],
                     bs_j[sample_indexes, orders[[2]]],
                     gs_normed[sample_indexes, orders[[1]]],
                     gs_normed[sample_indexes, orders[[2]]],
                     col = plotCol)
        }

        # Cluster 2+
        if (length(clust_indexes) > 1) {
            for (i in seq(2, length(clust_indexes))) {
                sample_indexes <- clust_indexes[[i]]
                points(gs_normed[sample_indexes, orders[[1]]],
                       gs_normed[sample_indexes, orders[[2]]],
                       col = plot_colors[[names(clust_indexes)[i]]],
                       pch = 16, cex = cex)

                for (j in seq(1, length(bs_normed))) {
                    bs_j <- bs_normed[[j]]
                    points(bs_j[sample_indexes, orders[[1]]],
                           bs_j[sample_indexes, orders[[2]]],
                           col = plot_colors[[names(clust_indexes)[i]]],
                           pch = j - 1, cex = cex)
                    segments(bs_j[sample_indexes, orders[[1]]],
                             bs_j[sample_indexes, orders[[2]]],
                             gs_normed[sample_indexes, orders[[1]]],
                             gs_normed[sample_indexes, orders[[2]]],
                             col = plot_colors[[names(clust_indexes)[i]]])
                }
            }
        }

        # Adding legend
        if (!tolower(legend_loc) == "none") {
            # plotting legend without clusters/categories
            if (length(plot_colors) == 1) {
                legend(legend_loc,
                       legend = c(names(mcia_results@block_loadings)),
                       pch = 0:length(mcia_results@block_loadings), cex = cex)

            # plotting legend for clusters/categories
            } else {
                leg_labels <- c(names(mcia_results@block_loadings),
                                names(plot_colors))
                leg_shapes <- c(seq(1, length(mcia_results@block_loadings)),
                                rep(16, length(plot_colors))) - 1
                leg_colors <- c(rep("black",
                                    length(mcia_results@block_loadings)),
                                unname(unlist(plot_colors)))

                legend(legend_loc, legend = leg_labels,
                       col = leg_colors, pch = leg_shapes, cex = cex)
            }
        }

    } else if (tolower(projection) == "block") {
        # Normalize global scores to unit variance
        # Still included to make comparable plots
        gs_norms <- apply(mcia_results@global_scores, 2,
                          function(x) (sqrt(var(x))))
        gs_normed <- t(t(mcia_results@global_scores) / gs_norms)

        # Check for the presence of the block name
        block_check <- any(block_name == names(mcia_results@block_scores))
        if (block_check == FALSE) {
            block_names <- paste(names(mcia_results@block_scores),
                                 collapse = ", ")
            msg <- paste0("block_name: '", block_name,
                          "' is not part of the data blocks list: ",
                          "'", block_names, "'.")
            stop(msg)
        }
        block_idx <- which(block_name == names(mcia_results@block_scores))
        block_idx <- as.numeric(block_idx)

        # Normalize block scores to unit variance
        # message(mcia_results@block_scores[[block_idx]])
        bs_norms <- apply(mcia_results@block_scores[[block_idx]], 2,
                          function(x) (sqrt(var(x))))
        bs_normed <- t(t(mcia_results@block_scores[[block_idx]]) / bs_norms)

        # Getting bounds for projection plot
        # minimum 1st block score
        min_bs1 <- min(bs_normed[, orders[[1]]])
        # minimum 2nd block score
        min_bs2 <- min(bs_normed[, orders[[2]]])
        # maximum 1st block score
        max_bs1 <- max(bs_normed[, orders[[1]]])
        # maximum 2nd block score
        max_bs2 <- max(bs_normed[, orders[[2]]])

        # minimum x coordinate in plot
        min_x <- min(c(min_bs1, min(gs_normed[, orders[[1]]])))
        # minimum y coordinate in plot
        min_y <- min(c(min_bs2, min(gs_normed[, orders[[2]]])))
        # maximum x coordinate in plot
        max_x <- max(c(max_bs1, max(gs_normed[, orders[[1]]])))
        # maximum y coordinate in plot
        max_y <- max(c(max_bs2, max(gs_normed[, orders[[2]]])))

        # Cluster 1
        sample_indexes <- clust_indexes[[1]]

        # catching error for one color
        if (is.null(color_col)) {
          plotCol <- plot_colors[[1]]
        } else {
          plotCol <- plot_colors[[names(clust_indexes)[1]]]
        }

        # Plotting block scores
        plot(bs_normed[sample_indexes, orders[[1]]],
             bs_normed[sample_indexes, orders[[2]]],
             main = "Factor Plot",
             xlab = paste("Factor", orders[[1]]),
             ylab = paste("Factor", orders[[2]]),
             col = plotCol,
             xlim = c(min_x, max_x),
             ylim = c(min_y, max_y),
             pch = 16, cex = cex)
        grid()

        # Cluster 2+
        if (length(clust_indexes) > 1) {
            for (i in seq(2, length(clust_indexes))) {
                sample_indexes <- clust_indexes[[i]]
                points(bs_normed[sample_indexes, orders[[1]]],
                       bs_normed[sample_indexes, orders[[2]]],
                       col = plot_colors[[names(clust_indexes)[i]]],
                       pch = j - 1, cex = cex)
            }
        }

        # Adding legend
        if (!tolower(legend_loc) == "none") {
            # plotting legend without clusters/categories
            if (length(plot_colors) == 1) {
                # do not plot legend when only one block is used
                invisible()

            # plotting legend for clusters/categories
            } else {
                leg_labels <- c(names(mcia_results@block_loadings),
                                names(plot_colors))
                leg_shapes <- c(seq(1, length(mcia_results@block_loadings)),
                                rep(16, length(plot_colors))) - 1
                leg_colors <- c(rep("black",
                                    length(mcia_results@block_loadings)),
                                unname(unlist(plot_colors)))
                legend(legend_loc, legend = leg_labels,
                       col = leg_colors, pch = leg_shapes, cex = cex)
            }
        }

    } else if (tolower(projection) == "global") {
        ### Plot 2 - projection plot global score only
        # Normalize global scores to unit variance
        gs_norms <- apply(mcia_results@global_scores, 2,
                          function(x) (sqrt(var(x))))
        gs_normed <- t(t(mcia_results@global_scores) / gs_norms)

        # Getting bounds for projection plot
        min_x <- min(gs_normed[, orders[[1]]]) # minimum x coordinate in plot
        min_y <- min(gs_normed[, orders[[2]]]) # minimum y coordinate in plot
        max_x <- max(gs_normed[, orders[[1]]]) # maximum x coordinate in plot
        max_y <- max(gs_normed[, orders[[2]]]) # maximum y coordinate in plot

        sample_indexes <- clust_indexes[[1]]

        # catching error for one color
        if (is.null(color_col)) {
          plotCol <- plot_colors[[1]]
        } else {
          plotCol <- plot_colors[[names(clust_indexes)[1]]]
        }

        plot(gs_normed[sample_indexes, orders[[1]]],
             gs_normed[sample_indexes, orders[[2]]],
             main = "Global Factor Plot",
             xlab = paste("Factor", orders[[1]]),
             ylab = paste("Factor", orders[[2]]),
             col = plotCol,
             xlim = c(min_x, max_x),
             ylim = c(min_y, max_y),
             pch = 16,
             cex = cex)
        grid()

        # Cluster 2+
        if (length(clust_indexes) > 1) {
            for (i in seq(2, length(clust_indexes))) {
                sample_indexes <- clust_indexes[[i]]
                points(gs_normed[sample_indexes, orders[[1]]],
                       gs_normed[sample_indexes, orders[[2]]],
                       col = plot_colors[[names(clust_indexes)[i]]],
                       pch = 16, cex = cex)
            }
        }

        # Adding legend
        # plotting legend for clusters/categories
        if (!is.null(color_col)) {
            leg_labels <- c(names(plot_colors))
            leg_shapes <- c(rep(16, length(plot_colors)))
            leg_colors <- c(unname(unlist(plot_colors)))
            legend(legend_loc, legend = leg_labels,
                   col = leg_colors, pch = leg_shapes, cex = cex)
        }
    }
}

# projection_plot uses base R plots

test_that("projection_plot: checking global figure", {
    expect_no_error(projection_plot(mcia_results, projection = "global",
                                    orders = c(1, 2)))
})

test_that("projection_plot: checking global figure with colors", {
    expect_no_error(projection_plot(mcia_results, projection = "global",
                                    orders = c(1, 2), color_col = "cancerType",
                                    color_pal = meta_colors,
                                    legend_loc = "bottomleft"))
})

# test_that("projection_plot: check color override", {
#     color_override <- setNames(scales::viridis_pal(option = "D")(3),
#                                nm = c("CNS", "Leukemia", "Melanoma"))
#
#     expect_no_error(projection_plot(mcia_results, projection = "global",
#                                     orders = c(1, 2), color_col = "cancerType",
#                                     color_pal = meta_colors,
#                                     legend_loc = "bottomleft",
#                                     color_override = color_override))
# })

test_that("projection_plot: check color column name existence", {
    expect_error(projection_plot(mcia_results, projection = "global",
                                 orders = c(1, 2), color_col = "CancerType",
                                 color_pal = meta_colors,
                                 legend_loc = "bottomleft"))
})

test_that("projection_plot: check color column validity", {
    expect_error(projection_plot(mcia_results, projection = "global",
                                 orders = c(1, 2), color_col = 123,
                                 color_pal = meta_colors,
                                 legend_loc = "bottomleft"))
})

test_that("projection_plot: checking all figures", {
    expect_no_error(projection_plot(mcia_results, projection = "all",
                                    orders = c(1, 2)))
})

test_that("projection_plot: checking block figure", {
    expect_no_error(projection_plot(mcia_results, projection = "block",
                                    block_name = "prot",
                                    orders = c(1, 2)))
})

test_that("projection_plot: checking block figure - fail", {
    expect_error(projection_plot(mcia_results, projection = "block",
                                 block_name = "faux-block",
                                 orders = c(1, 2)))
    dev.off()
})

#' Create an MAE object from a list of data matrices and column data
#'
#' @description Create an MAE object from a set of data matrices and column data.
#'
#' @details Requires that sample names match across experiments and are identical
#' to primary names, will only convert data matrices to SummarizedExperiment class.
#' If the data is more complex, please follow the guidelines
#' for creating an MAE object outlined in `help(MultiAssayExperiment)`
#'
#' @param matrix_list named list of data matrices
#' @param row_format for lists of data frames, indicates whether rows of
#' datasets denote `feature` (default) or `sample`.
#' @param colData_input a data frame containing sample metadata; sample names
#' in the rownames should correspond to samples names in `matrix_list`
#' @return List of harmonized data matrices for input into nipals_multiblock()
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' @importFrom SummarizedExperiment SummarizedExperiment
#' @importClassesFrom MultiAssayExperiment MultiAssayExperiment
#' @export

simple_mae <- function(matrix_list, row_format = "feature",
                       colData_input = NULL) {
  # The MAE class takes in data matrices in feature x sample orientation
  if (row_format == "sample") {
    matrix_list <- lapply(matrix_list, t)
  } else if (row_format != "feature") {
    stop("Choose from sample or feature orientation")
  }

  # Check if sample names are missing
  for (i in seq_along(matrix_list)) {
    if (is.null(colnames(matrix_list[[i]]))) {
      stop("All omics must have sample names")
    }
  }

  if (is.null(colData_input)) {
    mae_object <-
      MultiAssayExperiment::MultiAssayExperiment(
        lapply(matrix_list, function(x) SummarizedExperiment(as.matrix(x))))
  }
  else {
    mae_object <-
      MultiAssayExperiment::MultiAssayExperiment(
        lapply(matrix_list, function(x) SummarizedExperiment(as.matrix(x))),
        colData = colData_input)
  }

  return(mae_object)
}

# test dimension of global loadings (two omics with equal features)
test_that("Number of block score weights correct (random)", {
  # test data with 3 samples, 2 omics, 5 features per omic
  omic1 <- data.frame((rand(3, 5)))
  omic2 <- data.frame((rand(3, 5)))
  rownames(omic1) <- rownames(omic2) <- c("s1", "s2", "s3")
  colnames(omic1) <- c("f11", "f21", "f31", "f41", "f51")
  colnames(omic2) <- c("f12", "f22", "f32", "f42", "f52")

  test_data <- list("omic1" = omic1, "omic2" = omic2)
  test_data_mae <- simple_mae(test_data, row_format = "sample")

  test <- nipals_multiblock(test_data_mae, plots = "none",
                            num_PCs = 4, tol = 1e-9)

  expect_equal(c(10, 4), dim(test@global_loadings))
})

test_that("Number of block score weights correct (NCI60)", {
  expect_equal(c(20448, 2), dim(mcia_results@global_loadings))
})

# testing magnitude of top three eigenvalues of NCI60 dataset
test_that("Top eigenvalues correct", {
  expect_equal(all(mcia_results3@eigvals > 0.21), TRUE)
})

# expect error if number of samples is wrong
test_that("Mismatched samples should throw error", {
  test_data <- list(omic1 = data.frame((rand(2, 5))),
                    omic2 = data.frame((rand(3, 5))))

  expect_error(nipals_multiblock(test_data, plots = "none",
                                 num_PCs = 4, tol = 1e-9))
})

sce1 <- matrix(rpois(100, lambda = 10), ncol = 10)
sce2 <- matrix(rpois(100, lambda = 10), ncol = 10)
sce_list <- list(sce1,sce2)

test_that("All omics must have sample names", {
    expect_error(simple_mae(sce_list))
})

test_that("Valid row format", {
    expect_error(simple_mae(sce_list, row_format = "nonsense"))
})

#' Assigning colors to different omics
#'
#' @description Creates a list of omics and associated colors for plotting. The
#' default palette was chosen to be color-blindness friendly.
#'
#' @param mcia_results object returned from nipals_multiblock() function
#' @param color_pal a function which returns color palettes (e.g. scales)
#' @param color_pal_params list of parameters for the corresponding function
#' @return List of omics with assigned colors
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                  plots = "none", tol = 1e-12)
#' colors_omics <- get_colors(mcia_results)
#' @importFrom scales viridis_pal
#' @importFrom methods is
#' @export
get_colors <- function(mcia_results, color_pal = scales::viridis_pal,
                       color_pal_params = list()) {
    omic_list <- names(mcia_results@block_loadings)

    if (is(color_pal, "function")) {
        colors_omics <- do.call(color_pal, color_pal_params)(length(omic_list))

    } else if (is(color_pal, "character")) {
        colors_omics <- color_pal
    }

    names(colors_omics) <- omic_list
    return(colors_omics)
}

# "data_blocks" (loaded with `data(NCI60)` during setup)
# has 3 types of omics and metadata

test_that("generate omics color palette", {
    expect_length(get_colors(mcia_results), 3)
})

# uses the default viridis palette
test_that("check omics color palette 1", {
    expect_equal(
        get_colors(mcia_results), # option = "D"
        c("mrna" = "#440154FF", "miRNA" = "#21908CFF", "prot" = "#FDE725FF")
    )
})

# tries a different type of viridis palette
test_that("check omics color palette 2", {
    expect_equal(
        get_colors(mcia_results, color_pal_params = list(option = "A")),
        c("mrna" = "#000004FF", "miRNA" = "#B63679FF", "prot" = "#FCFDBFFF")
    )
})

# tries a different type of palette
test_that("check omics color palette 3", {
    expect_equal(
        get_colors(mcia_results, color_pal = scales::hue_pal),
        c("mrna" = "#F8766D", "miRNA" = "#00BA38", "prot" = "#619CFF")
    )
})

# tries a different type of palette with a different option
test_that("check omics color palette 4", {
    expect_equal(
        get_colors(mcia_results, color_pal = scales::hue_pal,
                   color_pal_params = list(h = c(0, 90))),
        c("mrna" = "#FF6C91", "miRNA" = "#DE8C00", "prot" = "#9DA700")
    )
})

test_that("use given omics color palette", {
    expect_length(get_colors(mcia_results,
                             color_pal = c("red", "green", "blue")), 3)
})

#' Computes the total variance of a multi-omics dataset
#'
#' @description Computes the total variances of all data blocks in a
#' multi-omics dataset, intended for datasets that do not use `CCpreproc`
#' @param ds a list of multi-omics dataframes/matrices in "sample x variable"
#' format
#' @return the total variance of the dataset (i.e. sum of block variances)
#' @examples
#' data(NCI60)
#' tot_var <- get_tv(data_blocks)
#' @export
get_tv <- function(ds) {
    ds <- lapply(ds, as.matrix)

    # function to calculate variances of each block
    get_bv <- function(X) {
        # comment out for centered matrices
        X_cent <- sweep(X, 2, colMeans(X), "-")

        n_samp <- dim(X_cent)[1]
        X_cent_norm <- X_cent

        bv <- (norm(X_cent_norm, type = "F")^2) / max(n_samp - 1, 1)
        return(bv)
    }

    bv_list <- lapply(ds, get_bv)
    # tv <- sum(unlist(bv_list))

    return(bv_list)
}

# calculate TV using long method
# mrna_c <- sweep(data_blocks$mrna, 2, colMeans(data_blocks$mrna), "-")
# mirna_c <- sweep(data_blocks$miRNA, 2, colMeans(data_blocks$miRNA), "-")
# prot_c <- sweep(data_blocks$prot, 2, colMeans(data_blocks$prot), "-")

# p1 <- prcomp(mrna_c, scale = FALSE)
# p1_s <- p1$sdev
# tv_p1 <- sum(p1_s^2)

# p2 <- prcomp(mirna_c,scale = FALSE)
# p2_s <- p2$sdev
# tv_p2 <- sum(p2_s^2)

# p3 <- prcomp(prot_c, scale = FALSE)
# p3_s <- p3$sdev
# tv_p3 <- sum(p3_s^2)

# tv_final <- list(mrna = tv_p1, mirna = tv_p2, prot = tv_p3)
tv_final <- list(mrna = 10152.24, miRNA = 1071.079, prot = 28843.05)

test_that("get_tv equals standard method", {
  expect_equal(get_tv(data_blocks), tv_final, tolerance = 1e-4)
})

#' Deflation via global scores
#'
#' @description Removes data from a data frame in the direction of a given
#' global scores vector.
#'
#' @details Subtracts the component of each column in the direction of a given
#' global scores vector to yield a `deflated' data matrix.
#'
#' @param df a data frame in "sample" x "variable" format
#' @param gs a global scores vector in sample space
#' @return the deflated data frame
#' @examples
#' df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
#' global_score <- rbinom(5, 1, prob = 0.3)
#' deflated_data <- deflate_block_gs(df, global_score)
#'
#' @export
deflate_block_gs <- function(df, gs) {
    normed_gs <- gs / norm(gs, type = "2")
    df <- df - tcrossprod(normed_gs) %*% as.matrix(df)
    return(df)
}

test_df <- cbind(c(0, 0, 1, 1, 0), c(1, 0, 0, 0, 0), c(0, 0, 0, 1, 1))
global_score <- c(0, 0, 1, 0, 0)

normed_gs <-  global_score / norm(global_score, type = "2")
deflate_gs <- test_df - tcrossprod(normed_gs) %*% as.matrix(test_df)

test_that("test output",  {
  expect_equal(deflate_gs, deflate_block_gs(test_df, global_score))
})

#' Perform biological annotation-based comparison
#'
#' @description Runs fgsea for the input gene vector
#' @param metagenes Vector of gene scores where the row names are HUGO symbols
#' @param path.database path to a GMT annotation file
#' @param factors vector of factors which should be analyzed
#' @param pval.thr p-value threshold (default to 0.05)
#' @param nproc number of processors to utilize
#' @return data frame with the most significant p-value number of significant
#' pathways
#' @return the selectivity scores across the given factors
#' @importFrom stats setNames
#' @export
gsea_report <- function(metagenes, path.database, factors = NULL,
                        pval.thr = 0.05, nproc = 4) {
    # Load annotation database
    pathways <- fgsea::gmtPathways(path.database)

    # Number of factors
    if (is.null(factors)) {
        factors <- seq_len(ncol(metagenes))
    }

    # Containers to report results
    report_min_pval <- numeric(0)
    report_number <- numeric(0)
    sig_paths <- numeric(0)

    # Calculate biological annotation enrichment, for each factor
    for (j in factors) {
        message("Running GSEA for Factor ", j)

        # Extract a vector of scores for GSEA and set rownames
        # to HUGO symbols
        scores <- setNames(as.matrix(metagenes[, j]), rownames(metagenes))

        # Compute GSEA
        fgseaRes <- fgsea::fgseaMultilevel(pathways, scores,
                                           nPermSimple = 10000, minSize = 15,
                                           maxSize = 500, nproc = nproc)

        # Report if at least one pathway is significant and
        # the min-pvalue from all pathways
        if (sum(fgseaRes$padj < pval.thr) != 0) {
            # Report the minimum p-value
            min_pval <- min(fgseaRes$padj)
            report_min_pval <- rbind(report_min_pval, min_pval)

            # Keep names of significant pathways
            curr_sig_paths <- fgseaRes[fgseaRes$padj < pval.thr, "pathway"]
            sig_paths <- c(sig_paths, curr_sig_paths)

            # Report number of unique significant pathways
            report_number <- rbind(report_number,
                                   dplyr::n_distinct(curr_sig_paths))

        } else {
            # Report the minimum p-value, assigning NA
            report_min_pval <- rbind(report_min_pval, NA)

            # Report number of unique significant pathways, assigning 0
            report_number <- rbind(report_number, 0)
        }
    }

    # Report selectivity
    # Selectivity is calculated best across ALL factors
    # The formula is give within https://go.nature.com/3EjW5HJ
    # and section "Selectivity Score". SS = (Nc + Nf) / 2L
    # Nc is the total number of clinical annotations associated with at
    # least a factor, Nf the total number of factors associated with at
    # least a clinical annotation, and L the total number of
    # associations between clinical annotations and factors. S has a
    # maximum value of 1 when each factor is associated with one and
    # only one clinical/biological annotation, and a minimum of 0 in
    # the opposite case. An optimal method should thus maximize its
    # number of factors associated with clinical/biological annotations
    # without having a too low selectivity.
    Nc <- dplyr::n_distinct(sig_paths)
    Nf <- length(which(!is.na(min_pval)))
    L <- length(sig_paths)
    SS <- (Nc + Nf) / (2 * L)

    # Setting up the final reporting data structure with list containing
    # two values, the first is a dataframe where the rows are factors and
    # the columns is composed of the following fields:
    # - the minimum p-value of all pathways
    # - the total number of significant pathways
    # Lastly, the second value is the selectivity value which assesses whether
    # the set of factors are capturing very different gene sets/pathways from
    # one another
    out <- list()
    out[[1]] <- data.frame(min_pval = report_min_pval,
                           total_pathways = report_number)
    row.names(out[[1]]) <- paste0("Factor", factors)

    out[[2]] <- SS
    names(out) <- c("per-factor-results", "selectivity")
    return(out)
}

# extract mRNA global loadings
mrna_gfscores <- mcia_results@global_loadings
mrna_rows <- str_detect(row.names(mrna_gfscores), "_mrna")
mrna_gfscores <- mrna_gfscores[mrna_rows, ]

# rename rows to contain HUGO based gene symbols
row.names(mrna_gfscores) <- str_remove(rownames(mrna_gfscores), "_[0-9]*_.*")

# load pathway data
path.database <- "https://data.broadinstitute.org/gsea-msigdb/msigdb/release/6.2/c2.cp.reactome.v6.2.symbols.gmt"
pathways <- fgsea::gmtPathways(gmt.file = path.database)

test_that("gsea_report", {
    # generate the GSEA report
    set.seed(10)
    geneset_report <- gsea_report(metagenes = mrna_gfscores, path.database,
                                  factors = seq(1), pval.thr = 0.05, nproc = 1)

    expect_equal(geneset_report$selectivity, 0.50423729, tolerance = 0.01)
    expect_equal(nrow(geneset_report$`per-factor-results`), 1)
    expect_equal(geneset_report$`per-factor-results`$min_pval[1], 2.120101e-09)
    expect_equal(geneset_report$`per-factor-results`$total_pathways[1], 118,
                 tolerance = 5)
})

#' An S4 class to contain results computed with `nipals_multiblock()`
#' @slot global_scores A matrix containing global scores as columns.
#' @slot global_loadings A matrix containing global loadings as columns.
#' @slot block_score_weights A matrix containing block weights as columns.
#' @slot block_scores A list of matrices. Each matrix contains the scores
#' as columns for a given block.
#' @slot block_loadings A list of matrices. Each matrix contains the
#' loadings as columns for a given block.
#' @slot eigvals A list of singular values of the data matrix at each
#' deflation step.
#' @slot col_preproc_method character for the column-level preprocessing
#' method used. See `col_preproc()`.
#' @slot block_preproc_method character for the block-level
#' preprocessing method used. See `block_preproc()`.
#' @slot block_variances A list of variances for each block.
#' @slot metadata A data frame of metadata originally passed into
#' `nipals_multiblock()`.
#'
#' @returns A NipalsResult object.
#' @exportClass NipalsResult

NipalsResult <- setClass("NipalsResult",
  representation(
    global_scores = "matrix",
    global_loadings = "matrix",
    block_score_weights = "matrix",
    block_scores = "list",
    block_loadings = "list",
    eigvals = "matrix",
    col_preproc_method = "character",
    block_preproc_method = "character",
    block_variances = "list",
    metadata = "data.frame")
)

# defining show method for NipalsResult
setMethod("show", "NipalsResult",
  function(object){
    cat(is(object)[[1]], " Object with properties: \n",
      "> Dataset dimensions:   ", nrow(object@global_scores), " x ",
      nrow(object@global_loadings), "\n",
      "> Number of blocks:  ", length(object@block_variances), "\n",
      "> Order of scores:  ", ncol(object@global_scores), "\n",
      "> Column preprocessing:  ", object@col_preproc_method, "\n",
      "> Block preprocessing:  ", object@block_preproc_method, "\n",
      "> Block names and sizes:   \n",
      sep = ""
    )
    blockdims <- lapply(object@block_loadings, nrow)
    print(unlist(blockdims))
  }
)

#' block_weights_heatmap
#'
#' @description Function to plot heatmap of block score weights
#'
#' @details Plotting function for heatmap of block score weights
#'
#' @param mcia_results MCIA results object returned from `nipals_multiblock`
#'
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                   plots = "none", tol = 1e-12)
#' block_weights_heatmap(mcia_results)
#' @return heatmap object containing the block weights as a heatmap
#' @export
block_weights_heatmap <- function(mcia_results) {
    bs_weights <- as.matrix(data.frame(mcia_results@block_score_weights))
    colnames(bs_weights) <- seq_len(ncol(bs_weights))

    ComplexHeatmap::Heatmap(matrix = bs_weights,
                            name = "Weight",
                            column_title = "Factors",
                            column_title_side = "bottom",
                            cluster_rows = FALSE,
                            cluster_columns = FALSE,
                            column_names_rot = 0,
                            column_names_centered = TRUE)
}

bw_heatmap <- block_weights_heatmap(mcia_results)
bs_weights <- as.matrix(data.frame(mcia_results@block_score_weights))
colnames(bs_weights) <- seq_len(ncol(bs_weights))

test_that("block_weights_heatmap: checking figure", {
    expect_no_error(block_weights_heatmap(mcia_results))
})

test_that("input is correct", {
  expect_equal(bw_heatmap@matrix, bs_weights)
})

#' Centered Column Profile Pre-processing
#'
#' @description Converts data blocks into centered column profiles where each
#' block has unit variance. Mimics the pre-processing in the Omicade4 package
#' (Meng et al. 2014)
#' @details Performs the following steps on a given data frame: \itemize{
#' \item Offsets data to make whole matrix non-negative
#' \item Divides each column by its sum
#' \item Subtracts (row sum/total sum) from each row
#' \item Multiplies each column by sqrt(column sum/total sum)
#' \item Divides the whole data frame by its total variance (the sqrt of the
#' sum of singular values)
#' }
#' @param df the data frame to apply pre-processing to, in "sample" x
#' "variable" format
#' @return the processed data frame
#' @examples
#' df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
#' preprocessed_dataframe <- cc_preproc(df)
#' @export
cc_preproc <- function(df) {
    temp_df <- as.matrix(df)

    # Making data non-negative
    min_val <- min(temp_df)
    if (min_val < 0) {
        offset <- floor(min_val)
        temp_df <- temp_df + abs(offset)
    }

    # Generating centered column profiles:
    totsum <- sum(temp_df)
    colsums <- colSums(temp_df)
    row_contribs <- rowSums(temp_df) / totsum

    # Dividing by column sums
    nz_cols <- which(colsums != 0) # excluding zero columns to avoid NaNs
    temp_df[, nz_cols] <- t(t(temp_df[, nz_cols]) / colsums[nz_cols])

    # Subtracting row contributions
    temp_df <- temp_df - row_contribs

    # Applying feature weighting ("multiplication by feature metrics")
    temp_df <- t(t(temp_df) * sqrt(colsums / totsum))

    # Weighting block to unit variance (block variance computed via Fro norm)
    block_var <- norm(temp_df, type = "F")^2 / (max(1, nrow(df) - 1))

    if (block_var != 0) {
        temp_df <- temp_df * (1 / sqrt(block_var))
    } else {
        warning("New data has zero variance.")
    }

    return(temp_df)
}

simple_input <- matrix(c(1, 0, 0, 0, 1, 0, 0, 1, 0), nrow = 3, ncol = 3)
temp_df <- simple_input

totsum <- 3
colsums <- c(1, 1, 1)
row_contribs  <-  c(1, 2, 0) / totsum

temp_df <- temp_df - row_contribs

temp_df <- t(t(temp_df) * sqrt(colsums / totsum))
block_var <- norm(temp_df, type = "F")^2 / 2
simple_input_cc_preproc <- temp_df / sqrt(block_var)


test_that("cc_preproc works on simple input", {
  expect_equal(simple_input_cc_preproc, cc_preproc(simple_input))
})

#' Visualize all loadings on two factor axes
#'
#' @description Visualize all loadings recovered from
#' nipalsMCIA() output loadings matrix ranked using across two factor axes
#'
#' @param mcia_results object returned from nipals_multiblock() function
#' @param axes list of two numbers associated with two factors to visualize
#' @param color_pal a list of colors or function which returns a list of colors
#' @param color_pal_params a list of parameters for the color function
#' @return Plot of MCIA feature loadings for chosen axes
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                   plots = "none", tol = 1e-12)
#' vis_load_plot(mcia_results, axes = c(1, 4))
#' @importFrom ggplot2 ggplot aes_string geom_point theme_bw scale_color_manual
#' @importFrom ggplot2 xlab ylab labs
#' @export
vis_load_plot <- function(mcia_results, axes = c(1, 2),
                          color_pal = scales::viridis_pal,
                          color_pal_params = list()) {

    # get the colors
    plot_colors <- get_colors(mcia_results,
                              color_pal = color_pal,
                              color_pal_params = color_pal_params)


    # extracting global loadings
    gl <- mcia_results@global_loadings

    omic_dims <- vapply(mcia_results@block_loadings, dim, numeric(2))[1, ]
    omic_type <- c()
    omics_labels <- names(mcia_results@block_loadings)
    for (i in seq_along(omics_labels)) {
        omic_label <- omics_labels[i]
        length_omic <- omic_dims[i]
        omic_type <- c(omic_type, rep(omic_label, each = length_omic))
    }

    gl_f <- data.frame(gl[, axes])
    gl_f$omic <- omic_type
    gl_f$omic <- as.factor(gl_f$omic)
    colnames(gl_f) <- c(paste0("Axis_", axes[1]),
                        paste0("Axis_", axes[2]), "omic")

    # plot data
    omic <- gl_f$omic
    ggplot(data = gl_f,
           aes(x = gl_f[[colnames(gl_f)[1]]], y = gl_f[[colnames(gl_f)[2]]],
               color = omic)) +
      geom_point(alpha = 0.3) +
      labs(x = paste0("Axis ", axes[1]), y = paste0("Axis ", axes[2])) +
         # color = "Omic"
      scale_color_manual(values = plot_colors) +
      theme_bw()
}

test_that("figure generation", {
    #dev.new()
    #par(mar = c(2, 2, 2, 2))
    expect_no_error(vis_load_plot(mcia_results, axes = c(1, 2)))
    #dev.off()
})

test_that("incorrect axes", {
    #dev.new()
    #par(mar = c(2, 2, 2, 2))
    expect_error(vis_load_plot(mcia_results, axes = c(1, 10)))
    #dev.off()
})

# when we add in a default color palette to this function, we should switch this to expect_no_error
test_that("color scheme", {
    expect_error(vis_load_plot(mcia_out = mcia_results, axes = c(1, 2)))
})

test_that("default axes", {
    expect_no_error(vis_load_plot(mcia_results))
})

#' Centered Column Profile Pre-processing
#'
#' @description Converts data blocks into centered column profiles where each
#' block has unit variance. Mimics the pre-processing in the Omicade4 package
#' (Meng et al. 2014)
#' @details Performs preprocessing on a sample/variable (row/column) level
#' according to the parameter given.
#' @param df the data frame to apply pre-processing to, in "sample" x
#' "variable" format
#' @param col_preproc_method denotes the type of column-centered preprocessing.
#' Options are: \itemize{
#' \item `colprofile` Performs the following steps on a given data frame:
#' \itemize{
#' \item Offsets data to make whole matrix non-negative
#' \item Divides each column by its sum
#' \item Subtracts (row sum/total sum) from each row
#' \item Multiplies each column by sqrt(column sum/total sum)
#' }
#' \item `standardized` centers each column and divides by its standard
#' deviation.
#' \item `centered_only` ONLY centers data
#' }
#' @return the processed data frame
#' @examples
#' df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
#' preprocessed_dataframe <- col_preproc(df, col_preproc_method = 'colprofile')
#' @export
col_preproc <- function(df, col_preproc_method) {
    temp_df <- as.matrix(df)
    col_preproc_method <- tolower(col_preproc_method)

    if (col_preproc_method == "colprofile") {
        # Making data non-negative
        min_val <- min(temp_df)
        if (min_val < 0) {
            offset <- floor(min_val)
            temp_df <- temp_df + abs(offset)
        }

        # Generating centered column profiles:
        totsum <- sum(temp_df)
        colsums <- colSums(temp_df)
        row_contribs <- rowSums(temp_df) / totsum

        # Dividing by column sums
        nz_cols <- which(colsums != 0) # excluding zero columns to avoid NaNs
        temp_df[, nz_cols] <- t(t(temp_df[, nz_cols]) / colsums[nz_cols])

        # Subtracting row contributions
        temp_df <- temp_df - row_contribs

        # Applying feature weighting by proportion of variance
        temp_df <- t(t(temp_df) * sqrt(colsums / totsum))

    } else if (col_preproc_method == "standardized") {
        temp_df <- scale(temp_df)
    } else if (col_preproc_method == "centered_only") {
        temp_df <- scale(temp_df, center = TRUE, scale = FALSE)
    } else {
        stop("Column preprocessing method not recognized ",
             "- pick from available options")
    }

    return(temp_df)
}

test_that("standardized column preprocessing", {
    prot_stand <- scale(as.matrix(prot), center = TRUE, scale = TRUE)

    expect_equal(col_preproc(prot, "standardized"), prot_stand)
})

test_that("centered only column preprocessing", {
    prot_centered <- scale(as.matrix(prot), center = TRUE, scale = FALSE)

    expect_equal(col_preproc(prot, "centered_only"), prot_centered)
})

#' NIPALS Iteration
#'
#' @description Applies one iteration stage/loop of the NIPALS algorithm.
#'
#' @details Follows the NIPALS algorithm as described by Hanafi et. al. (2010).
#' Starts with a random vector in sample space and repeatedly projects it onto
#' the variable vectors and block scores to generate block and global
#' loadings/scores/weights. The loop stops when either the stopping criterion
#' is low enough, or the maximum number of iterations is reached. Intended as a
#' utility function for `nipals_multiblock` to be used between deflation steps.
#'
#' @param ds a list of data matrices, each in "sample" x "variable" format
#' @param tol a number for the tolerance on the stopping criterion for NIPALS
#' @param maxIter a number for the maximum number of times NIPALS should iterate
#' @return a list containing the global/block scores, loadings and weights for
#' a given order
#' @examples
#' data(NCI60)
#' data_blocks <- lapply(data_blocks, as.matrix)
#' nipals_results <- nipals_iter(data_blocks, tol = 1e-7, maxIter = 1000)
#' @importFrom pracma rand
#' @importFrom RSpectra svds
#' @importFrom stats cov var
#' @export
nipals_iter <- function(ds, tol = 1e-12, maxIter = 1000) {
    # Main iteration loop
    stop_crit <- 2 * tol
    cov_squared_old <- 0
    iter <- 0
    gs <- pracma::rand(nrow(ds[[1]]), 1) # begin with random global score vector

    while (stop_crit > tol && iter <= maxIter) {
        # Computing block loadings
        bl_list <- lapply(ds, function(df, q) {
            bl_k <- crossprod(df, q)
            bl_k <- bl_k / norm(bl_k, type = "2")
            return(bl_k)
        }, q = gs)

        # Computing block scores
        bs_list <- mapply(function(df, bl_k) {
            bs_k <- df %*% bl_k
            return(bs_k)
        }, ds, bl_list)

        # Computing global weights
        gw <- crossprod(bs_list, gs)
        gw <- gw / norm(gw, type = "2")
        gs <- bs_list %*% gw

        # Computing stopping criteria
        cov_list <- vapply(as.data.frame(bs_list), function(bs, gs) {
            gs_norm <- gs / sqrt(drop(var(gs)))
            return(drop(cov(bs, gs_norm))^2)
        }, gs = gs, FUN.VALUE = numeric(1))

        stop_crit <- abs(sum(cov_list) - cov_squared_old)
        cov_squared_old <- sum(cov_list)

        iter <- iter + 1
    }

    if (iter > maxIter) {
        warning("NIPALS iteration did not converge")
    }
  
    # Compute variance directly as 'eigenvalue'
    eigval <- var(gs)


    # Computing global loadings at final iteration
    gl <- bl_list[[1]] * gw[1]
    names(gl) <- rownames(bl_list[[1]])
    nblocks <- length(ds)
    for (i in seq(2, nblocks)) {
        temp_names <- c(names(gl), rownames(bl_list[[i]]))
        gl <- c(gl, bl_list[[i]] * gw[i])
        names(gl) <- temp_names
    }

    # Returning results
    retlist <- list(gs, gl, gw, bs_list, bl_list, eigval)
    names(retlist) <- c("global_scores", "global_loadings",
                        "block_score_weights", "block_scores",
                        "block_loadings", "eigval")
    return(retlist)
}

#' @name metadata_NCI60
#' @title NCI-60 Multi-Omics Metadata
#' @description Metadata for the included multi-omics dataset, denoting
#' the cancer type associated with each of the 21 cell lines.
#'
#' @references \url{https://github.com/aedin/NCI60Example}
#' @return List with 21 elements
#' @source Meng et. al, 2016 supplementary materials {https://doi.org/10.1093/bib/bbv108}
#' @keywords data multi-omics
NULL

#' Main NIPALS computation loop
#'
#' @description Applies the full adjusted NIPALS algorithm to generate block
#' and global scores/loadings with the desired deflation method.
#'
#' @details Follows the NIPALS algorithm as described by Hanafi et. al. (2010).
#' For each order of scores/loadings, the vectors are computed via the
#' `nipals_iter` function, then used to deflate the data matrix according to
#' the desired deflation method.
#' This process is repeated up to the desired maximum order of scores/loadings.
#'
#' @param data_blocks_mae a MultiAssayExperiment class
#' object (with sample metadata as a dataframe in the colData attribute).
#' @param col_preproc_method an option for the desired column-level data
#' pre-processing, either:
#' \itemize{
#' \item `colprofile` applies column-centering, row and column weighting by
#' contribution to variance.
#' \item `standardized` centers each column and divides by its standard
#' deviation.
#' \item `centered_only` ONLY centers data
#' }
#' @param block_preproc_method an option for the desired block-level data
#' pre-processing, either:\itemize{
#' \item `unit_var` FOR CENTERED MATRICES ONLY - divides each block by the
#' square root of its variance
#' \item `num_cols` divides each block by the number of variables in the block.
#' \item `largest_sv` divides each block by its largest singular value.
#' \item `none` performs no preprocessing
#' }
#' @param num_PCs the maximum order of scores/loadings
#' @param tol a number for the tolerance on the stopping criterion for NIPALS
#' @param max_iter a number for the maximum number of times NIPALS should
#' iterate
#' @param color_col Optional argument with the column name of the `metadata`
#' data frame used to define plotting colors
#' @param deflationMethod an option for the desired deflation method, either:
#' \itemize{
#' \item `block` deflation via block loadings (for MCIA, default)
#' \item `global` deflation via global scores (for CPCA)
#' }
#' @param plots an option to display various plots of results: \itemize{
#' \item `all` displays plots of block scores, global scores, and eigenvalue
#' scree plot
#' \item `global` displays only global score projections and eigenvalue scree
#' plot
#' \item `none` does not display plots
#' }
#' @param harmonize  boolean whether or not samples should be checked for duplicates 
#' and re-ordered so that each row corresponds to the same sample across datasets.
#' Set to FALSE to greatly reduce computation time on many samples (default = TRUE).
#' @return a `nipalsResult` object with the following fields: \itemize{
#' \item `global_scores` a matrix containing global scores as columns
#' (NOT normalized to unit variance)
#' \item `global_loadings` a matrix containing global loadings as columns
#' \item `global_score_weights` a matrix of weights to express global scores as
#' a combination of block scores. Has dimensions "num_Blocks" by "num_PCs"
#' \item `eigvals` a matrix containing the eigenvalue for each computed global
#' score.
#' \item `block scores` a list of matrices, each contains the scores for one
#' block
#' \item `block loadings` a list of matrices, each contains the loadings for
#' one block (w/ unit length)
#' \item `block score weights` a matrix containing weights for each block score
#' of each order used to construct the global scores.
#' \item `col_preproc_method` the column preprocessing method used on the data.
#' \item `block_variances` a list of variances of each block AFTER
#' NORMALIZATION OPTION APPLIED
#' \item `metadata` the metadata dataframe supplied with the `metadata`
#' argument. Note: overrides metadata present in any MAE class object.
#'}
#' @importFrom graphics par
#' @importFrom methods new
#' @importFrom MultiAssayExperiment colData
#' @importClassesFrom MultiAssayExperiment MultiAssayExperiment
#' @examples
#'    data(NCI60)
#'    data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                                  colData = metadata_NCI60)
#'    NIPALS_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                        tol = 1e-12, max_iter = 1000,
#'                                        col_preproc_method = "colprofile",
#'                                        deflationMethod = "block")
#'    MCIA_results <- nipals_multiblock(data_blocks_mae, num_PCs = 4)
#'    CPCA_results <- nipals_multiblock(data_blocks_mae, num_PCs = 4,
#'    deflationMethod = "global")
#'
#' @export
nipals_multiblock <- function(data_blocks_mae,
                              col_preproc_method = "colprofile",
                              block_preproc_method = "unit_var",
                              num_PCs = 10, tol = 1e-9,
                              max_iter = 1000,color_col = NULL,
                              deflationMethod = "block", plots = "all",
                              harmonize = TRUE) {
    # Check for input type MAE or list
    if (is(data_blocks_mae, "MultiAssayExperiment")) {
      data_blocks <- extract_from_mae(data_blocks_mae, harmonize = harmonize)
      metadata <- data.frame(MultiAssayExperiment::colData(data_blocks_mae))
      if (length(metadata) == 0) {
        metadata <- NULL
      }
    } else {
        stop("Unknown input data format - please use MultiAssayExperiment")
    }

    num_blocks <- length(data_blocks)
    omics_names <- names(data_blocks)

    # Check for omics names and assign generic name if null
    if (is.null(omics_names)) {
        omics_names <- paste("omic", seq(1, length(data_blocks)), sep = "")
        names(data_blocks) <- omics_names
    }

    # Check for feature names and assign generic name if null
    for (i in seq_along(data_blocks)) {
        fNames <- names(data_blocks[[i]]) # feature names

        # Error catching for no names - creates default values
        if (is.null(fNames) || nchar(fNames[1]) == 0) {
            fNames <- paste("feature", seq(1, dim(data_blocks[[i]])[[2]]),
                            sep = "_")
        }
    }

    # Pre-processing data
    message("Performing column-level pre-processing...")
    data_blocks <- lapply(data_blocks, col_preproc, col_preproc_method)
    message("Column pre-processing completed.")

    # Block-level pre-processing
    message("Performing block-level preprocessing...")
    data_blocks <- lapply(data_blocks, block_preproc, block_preproc_method)
    if (tolower(block_preproc_method) == "unit_var") {
        block_vars <- rep(list(1), length(data_blocks))
        names(block_vars) <- names(data_blocks)

    } else {
        block_vars <- get_tv(data_blocks)
    }

    message("Block pre-processing completed.")

    # First NIPALS run
    message("Computing order ", 1, " scores")
    nipals_result <- nipals_iter(data_blocks, tol)

    # Saving result
    # matrix containing global scores as columns
    global_scores <- nipals_result$global_scores
    # matrix containing global loadings as columns
    global_loadings <- as.matrix(nipals_result$global_loadings)
    # matrix containing block score weights as columns
    block_score_weights <- nipals_result$block_score_weights

    block_scores <- list() # list containing matrices of block scores
    block_loadings <- list() # list containing matrices of block loadings
    for (i in seq_along(data_blocks)) {
        block_scores[[i]] <- nipals_result$block_scores[, i]
        block_loadings[[i]] <- nipals_result$block_loadings[[i]]
    }

    # Computing block eigenvalue
    eigvals <- list(nipals_result$eigval)

    if (num_PCs > 1) {
        # generate scores/loadings up to number of PCs
        for (i in seq(2, num_PCs)) {
            message("Computing order ", i, " scores")
            # Deflate blocks
            if (tolower(deflationMethod) == "block") {
                data_blocks <- mapply(deflate_block_bl,
                                      data_blocks,
                                      nipals_result$block_loadings,
                                      SIMPLIFY = FALSE)

            } else if (tolower(deflationMethod) == "global") {
                data_blocks <- lapply(data_blocks,
                                      deflate_block_gs,
                                      gs = nipals_result$global_scores)

            } else {
                stop("Unknown option for deflation step -
                     use 'block' or 'global'")
            }

            # Run another NIPALS iteration
            nipals_result <- nipals_iter(data_blocks, tol)

            # Save results
            global_scores <- cbind(global_scores, nipals_result$global_scores)
            global_loadings <- cbind(global_loadings,
                                     nipals_result$global_loadings)
            block_score_weights <- cbind(block_score_weights,
                                         nipals_result$block_score_weights)
            eigvals <- cbind(eigvals, nipals_result$eigval)

            for (j in seq_along(block_scores)) {
                block_scores[[j]] <- cbind(block_scores[[j]],
                                           nipals_result$block_scores[, j])
                block_loadings[[j]] <- cbind(block_loadings[[j]],
                                             nipals_result$block_loadings[[j]])
            }
        }
    }

    # Formatting results
    # adding block names in outputs
    names(block_scores) <- names(data_blocks)
    names(block_loadings) <- names(data_blocks)
    names(eigvals) <- paste("gs", seq(1, num_PCs), sep = "")
    eigvals <- as.matrix(eigvals)

    # adding row (sample) names in outputs
    rownames(global_scores) <- rownames(data_blocks[[1]])

    for (j in seq_along(num_blocks)) {
        block_scores[[j]] <- as.matrix(block_scores[[j]])
        rownames(block_scores[[j]]) <- rownames(data_blocks[[1]])
    }

    # fixing no metadata with empty data frame
    if (is.null(metadata)) {
        metadata <- data.frame()
    }

    # creating S4 class object with nipals outputs
    mcia_out <- new("NipalsResult",
                    global_scores = global_scores,
                    global_loadings = global_loadings,
                    block_score_weights = block_score_weights,
                    block_scores = block_scores,
                    block_loadings = block_loadings,
                    eigvals = eigvals,
                    col_preproc_method = tolower(col_preproc_method),
                    block_preproc_method = tolower(block_preproc_method),
                    block_variances = block_vars,
                    metadata = metadata)

    # Plotting results
    if (tolower(plots) == "all" && num_PCs >1) {
        par(mfrow = c(1, 2))
        projection_plot(mcia_out, "all", legend_loc = "bottomleft",
                        color_col = color_col) # first two orders of scores
        global_scores_eigenvalues_plot(mcia_out) # global score eigenvalues
        par(mfrow = c(1, 1))

    } else if (tolower(plots) == "global"&& num_PCs >1) {
        par(mfrow = c(1, 2))
        # first two global scores
        projection_plot(mcia_out, "global", color_col = color_col)
        global_scores_eigenvalues_plot(mcia_out) # global score eigenvalues
        par(mfrow = c(1, 1))

    } else if (tolower(plots) == "none") {

    } else {
        message("No known plotting option (or numPCs=1) - skipping plots.")
    }

    return(mcia_out)
}

library(stringr)

# load in "data_blocks" and "metadata_NCI60"
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
                              colData = metadata_NCI60)

# extract just the proteomics dataset
prot <- data_blocks$prot
# prot <- as.matrix(prot) # standardized proteomics dataset

# run MCIA with different numbers of scores
mcia_results <- nipals_multiblock(data_blocks_mae,
                                  col_preproc_method = "colprofile",
                                  num_PCs = 2, tol = 1e-12, plots = "none")

mcia_results3 <- nipals_multiblock(data_blocks_mae,
                                   col_preproc_method = "colprofile",
                                   num_PCs = 3, tol = 1e-12, plots = "none")

# run MCIA without metadata, used for testing the graphing section
mcia_results_no_meta <- nipals_multiblock(data_blocks_mae,
                                          col_preproc_method = "colprofile",
                                          num_PCs = 2, tol = 1e-12,
                                          plots = "none")

# creating a shared set of meta colors
meta_colors <- get_metadata_colors(mcia_results, color_col = 1,
                                   color_pal_params = list(option = "E"))

# creating a shared set of omics colors
colors_omics <- get_colors(mcia_results)

#' Extract a list of harmonized data matrices from an MAE object
#'
#' @description Extract a list of harmonized data matrices for
#' input into nipals_multiblock() from an MAE object
#'
#' @param MAE_object an MAE object containing experiment data for extraction
#' colData field optional
#' experiments should either be SummarizedExperiment, SingleCellExperiment, or
#' RangedSummarizedExperiment classes
#' @param subset_data \itemize{
#' \item `all` use all experiments in MAE object
#' \item `c(omic1,omic2,...)` list of omics from names(MAE_object)
#' }
#' @param harmonize A boolean whether samples should be checked for duplicates \itemize{
#' \item `TRUE` (default) merges duplicate samples via the `MultiAssayExperiment::mergeReplicates` function
#' \item `FALSE` skips sample duplicate check - USE THIS FOR LARGE-SAMPLE DATASETS.
#' }
#' @return List of harmonized data matrices for input into `nipals_multiblock()`
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' NCI60_input = extract_from_mae(data_blocks_mae, subset = "all")
#' @importFrom MultiAssayExperiment colData assays experiments
#' @importFrom MultiAssayExperiment mergeReplicates intersectColumns
#' @importClassesFrom MultiAssayExperiment MultiAssayExperiment
#' @export


extract_from_mae <- function(MAE_object, subset_data = "all", harmonize = TRUE) {
  if (subset_data != "all") {
    if (intersect(subset_data, names(MAE_object)) < 2) {
      stop("Please provide appropriate subset_data list")
    }
    else {
      MAE_object <- MAE_object[,,subset]
    }
  }

  n_omics <- length(MAE_object)
  if (n_omics == 1) {
    stop("The nipalsMCIA algorithm is designed for analysis of > 1 omics experiments")
  }
  else {
    for (i in seq_along(c(1:n_omics))) {
      if (is.null(colnames(experiments(MAE_object)[[i]]))) {
        stop("All experiments must have colnames (sample names)")
      }
    }
  }

  if(harmonize){
    MAE_object <-
      MultiAssayExperiment::mergeReplicates(intersectColumns(MAE_object))
  }
  
  # Extract data matrices
  extracted_data <- MultiAssayExperiment::assays(MAE_object)@listData

  # Match experiment sample names to primary names if colData available
  if (length(colData(MAE_object)) > 0) {
    primary_names <- rownames(colData(MAE_object))

    for (i in seq_along(extracted_data)) {
      colnames(extracted_data[[i]]) <- primary_names
    }
  }
  rm(MAE_object)

  # Transpose to sample x feature format for input into MCIA
  extracted_data <- lapply(extracted_data, t)

  return(extracted_data)
}

test_that("The nipalsMCIA algorithm is designed for analysis of > 1 omics experiments",
          {expect_error(extract_from_mae(data_blocks_mae, subset = "prot"))
})

test_that("Please provide appropriate subset_data list",
          {expect_error(extract_from_mae(data_blocks_mae, subset = "chrom"))})

#' Assigning colors to different values of a metadata column
#'
#' @description Creates a list of metadata columns and associated colors
#' for plotting. The default palette was chosen to be color-blindness friendly.
#' @param mcia_results object returned from nipals_multiblock() function
#' @param color_col an integer or string specifying the column that will be
#'     used for color_col
#' @param color_pal a function which returns color palettes (e.g. scales)
#' @param color_pal_params list of parameters for the corresponding function
#' @return List of metadata columns with assigned colors
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                                colData = metadata_NCI60)
#' mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                   plots = "none", tol = 1e-12)
#' colors_omics <- get_metadata_colors(mcia_results, "cancerType",
#'                                     color_pal_params = list(option = "E"))
#' @importFrom scales viridis_pal
#' @export
get_metadata_colors <- function(mcia_results, color_col,
                                color_pal = scales::viridis_pal,
                                color_pal_params = list()) {
  meta_list <- unique(mcia_results@metadata[, color_col])
  meta_list <- sort(meta_list) # alphabetize the metadata

  if (is(color_pal, "function")) {
    colors_meta <- do.call(color_pal, color_pal_params)(length(meta_list))
  } else if (is(color_pal, "character")) {
    colors_meta <- color_pal
  }

  names(colors_meta) <- meta_list
  return(colors_meta)
}

# uses the default viridis palette
test_that("generate metadata color palette", {
  expect_length(get_metadata_colors(mcia_results), 3)
})

# tries a different type of palette
test_that("check metadata color palette", {
  expect_equal(
    get_metadata_colors(mcia_results, color_pal = scales::brewer_pal,
                        color_pal_params = list()),
    c("CNS" = "#DEEBF7", "Leukemia" = "#9ECAE1", "Melanoma" = "#3182BD")
  )
})

# tries a different type of palette with a different option
test_that("check metadata color palette", {
  expect_equal(
    get_metadata_colors(mcia_results, color_pal = scales::brewer_pal,
                        color_pal_params = list(palette = "Greens",
                                                direction = -1)),
    c("CNS" = "#31A354", "Leukemia" = "#A1D99B", "Melanoma" = "#E5F5E0")
  )
})

#' Accessor function for global scores
#'
#' @description Retrieves the global scores as a matrix from a `NipalsResult`
#' object, typically output from `nipals_multiblock()`.
#' @param nmb_object A `NipalsResult` object.
#' @return a matrix containing global scores.
#' @examples
#' data("NCI60")
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
#' global_scores <- nmb_get_gs(mcia_out)
#' @export

nmb_get_gs <- function(nmb_object) {
    return(nmb_object@global_scores)
}


#' Accessor function for global loadings
#'
#' @description Retrieves the global loadings as a matrix from a `NipalsResult`
#' object, typically output from `nipals_multiblock()`.
#' @param nmb_object A `NipalsResult` object.
#' @return a matrix containing global loadings.
#' @examples
#' data("NCI60")
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
#' global_loadings <- nmb_get_gl(mcia_out)
#' @export

nmb_get_gl <- function(nmb_object) {
    return(nmb_object@global_loadings)
}


#' Accessor function for block scores
#'
#' @description Retrieves the block scores as a list of matrices
#' from a `NipalsResult` object, typically output from `nipals_multiblock()`.
#' @param nmb_object A `NipalsResult` object.
#' @return a list of matrices containing block scores.
#' @examples
#' data("NCI60")
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
#' block_scores <- nmb_get_bs(mcia_out)
#' @export

nmb_get_bs <- function(nmb_object) {
    return(nmb_object@block_scores)
}


#' Accessor function for block loadings
#'
#' @description Retrieves the block loadings as a list of matrices
#' from a `NipalsResult` object, typically output from `nipals_multiblock()`.
#' @param nmb_object A `NipalsResult` object.
#' @return a list of matrices containing block loadings.
#' @examples
#' data("NCI60")
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
#' block_loadings<- nmb_get_bl(mcia_out)
#' @export

nmb_get_bl <- function(nmb_object) {
    return(nmb_object@block_loadings)
}


#' Accessor function for block score weights
#'
#' @description Retrieves the block score weights
#' from a `NipalsResult` object, typically output from `nipals_multiblock()`.
#' @param nmb_object A `NipalsResult` object.
#' @return a matrix containing the block score weights.
#' @examples
#' data("NCI60")
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
#' block_score_weights <- nmb_get_bs_weights(mcia_out)
#' @export

nmb_get_bs_weights <- function(nmb_object) {
    return(nmb_object@block_score_weights)
}

#' Accessor function for eigenvalues
#'
#' @description Retrieves the eigenvalues
#' from a `NipalsResult` object, typically output from `nipals_multiblock()`.
#' @param nmb_object A `NipalsResult` object.
#' @return a matrix containing the eigenvalues for all global scores.
#' @examples
#' data("NCI60")
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
#' nipals_eigvals <- nmb_get_eigs(mcia_out)
#' @export

nmb_get_eigs <- function(nmb_object) {
    return(nmb_object@eigvals)
}

#' Accessor function for metadata
#'
#' @description Retrieves the metadata
#' from a `NipalsResult` object, typically output from `nipals_multiblock()`.
#' @param nmb_object A `NipalsResult` object.
#' @return a dataframe containing metadata associated with the `NipalsResult` object.
#' @examples
#' data("NCI60")
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
#' nipals_metadata <- nmb_get_metadata(mcia_out)
#' @export

nmb_get_metadata <- function(nmb_object) {
  return(nmb_object@metadata)
}

#' @name data_blocks
#' @title NCI-60 Multi-Omics Data
#' @description A  dataset of measurements of 12,895 mRNA, 537 miRNA,
#' and 7,016 protein variables (columns) on 21 cancer cell lines (rows) from
#' the NCI-60 cancer cell line database.
#'
#' @references \url{https://github.com/aedin/NCI60Example}
#' @return Large list with 3 elements (one for each omic)
#' @source Meng et. al, 2016 supplementary materials {https://doi.org/10.1093/bib/bbv108}
#' @keywords data multi-omics
NULL

#' Deflation via block loadings
#'
#' @description Removes data from a data frame in the direction of a given
#' block loadings vector.
#'
#' @details Subtracts the component of each row in the direction of a given
#' block loadings vector to yield a `deflated' data matrix.
#'
#' @param df a data frame in "sample" x "variable" format
#' @param bl a block loadings vector in variable space
#' @return the deflated data frame
#' @examples
#' df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
#' block_loading <- rbinom(3, 1, prob = 0.3)
#' deflated_data <- deflate_block_bl(df, block_loading)
#'
#' @export
deflate_block_bl <- function(df, bl) {
    df <- df - tcrossprod(as.matrix(df) %*% bl, bl)
    return(df)
}

#' Prediction of new global scores based on block loadings and weights
#'
#' @description Uses previously-computed block scores and weights to compute
#' a global score for new data. Only validated for MCIA results, as CPCA loadings
#' aren't compatible with un-deflated data.
#'
#' @details Projects the new observations onto each block loadings vector, then
#' weights the projection according to the corresponding block weights.
#'
#' @param mcia_results an mcia object output by nipals_multiblock()
#'         containing block scores, weights, and pre-processing identifier.
#' @param test_data an MAE object with the same block types and features as the
#'         training dataset. Feature and omic order must match `bl`.
#' @return a matrix of predicted global scores for the training data
#' @examples
#'    data(NCI60)
#'    data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                                  colData = metadata_NCI60)
#'    mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 2)
#'    new_data <- data_blocks_mae # should update with a truly new dataset
#'    preds <- predict_gs(mcia_results, new_data)
#'
#' @export
#'
predict_gs <- function(mcia_results, test_data) {
    bl <- mcia_results@block_loadings
    bw <- mcia_results@block_score_weights
    col_preproc_method <- mcia_results@col_preproc_method
    block_preproc_method <- mcia_results@block_preproc_method

    # extract data from train MAE object
    df <- extract_from_mae(test_data)

    num_omics <- length(bl)
    if (length(df) != length(bl) || length(df) != dim(bw)[[1]]) {
        stop("Mismatched number of omics between the block loadings, ",
             "block weights, and new data.")
    }

    # Apply same pre-processing methods as the model
    message("Performing pre-processing on data")
    df <- lapply(df, col_preproc, col_preproc_method)
    df <- lapply(df, block_preproc, block_preproc_method)
    message("Pre-processing completed")

    # ensuring all arguments are matrices
    bl <- lapply(bl, as.matrix)

    if (dim(df[[1]])[[2]] != dim(bl[[1]])[[1]]) {
        stop("Mismatched number of features in omic ", 1)
    }

    new_gs <- df[[1]] %*% bl[[1]] # block score matrix for 1st omic
    new_gs <- t(t(new_gs) * bw[1, ]) # applying block weight

    # for each omics type
    if (num_omics > 1) {
        for (i in 2:num_omics) {
            if (dim(df[[i]])[[2]] != dim(bl[[i]])[[1]]) {
                stop("Mismatched number of features in omic ", i)
            }

            new_gs_i <- df[[i]] %*% bl[[i]] # block score matrix for ith omic
            new_gs <- new_gs + t(t(new_gs_i) * bw[i, ]) # applying block weight
        }
    }

    return(new_gs)
}

#' global_scores_eigenvalues_plot
#'
#' @description Function to plot eigenvalues of scores up to num_PCs
#'
#' @details Plotting function for eigenvalues of scores up to num_PCs
#'
#' @param mcia_results MCIA results object returned from `nipals_multiblock`
#'
#' @examples
#' data(NCI60)
#' data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
#'                               colData = metadata_NCI60)
#' mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
#'                                   plots = "none", tol = 1e-12)
#' global_scores_eigenvalues_plot(mcia_results)
#' @return Displays the contribution plot using eigenvalues
#' @importFrom graphics barplot
#' @export
global_scores_eigenvalues_plot <- function(mcia_results) {
    # getting total variance if supplied
    if (is.list(mcia_results@block_variances)) {
        ylabel <- "Prop. Total Variance"
        totvar <- sum(unlist(mcia_results@block_variances))

    } else {
        ylabel <- "Eigenvalue"
        totvar <- 1
    }

    # extract eigenvalues
    barploteigs <- (unlist(mcia_results@eigvals)) / totvar
    names(barploteigs) <- seq(1, length(mcia_results@eigvals))

    # generate barplot
    barplot(barploteigs, xlab = "Factors", ylab = ylabel, cex.names = 1,
            main = "Global Eigenvalues")
}

---
title: "Predicting New MCIA scores"
author:
- Max Mattessich
- Joaquin Reyna
- Edel Aron
- Anna Konstorum
date: "Compiled: `r format(Sys.time(), '%B %d, %Y')`"
output:
  BiocStyle::html_document:
    dev: 'jpeg'
    fig_retina: 1
    number_sections: FALSE
    toc_depth: 2
    toc_float: TRUE
vignette: >
  %\VignetteIndexEntry{Predicting New MCIA scores}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Predicting MCIA global (factor) scores for new test samples

It may be of interest to use the embedding that is calculated on a training
sample set to predict scores on a test set (or, equivalently, on new data).

After loading the `nipalsMCIA` library, we randomly split the NCI60 cancer cell
line data into training and test sets.

## Installation

```{r installation-github, eval = FALSE}
# devel version

# install.packages("devtools")
devtools::install_github("Muunraker/nipalsMCIA", ref = "devel",
                         force = TRUE, build_vignettes = TRUE) # devel version
```

```{r installation-bioconductor, eval = FALSE}
# release version
if (!require("BiocManager", quietly = TRUE))
  install.packages("BiocManager")

BiocManager::install("nipalsMCIA")
```

```{r load-packages, message = FALSE}
library(ggplot2)
library(MultiAssayExperiment)
library(nipalsMCIA)
```

## Split the data

```{r split-data}
data(NCI60)
set.seed(8)

num_samples <- dim(data_blocks[[1]])[1]
num_train <- round(num_samples * 0.7, 0)
train_samples <- sample.int(num_samples, num_train)

data_blocks_train <- data_blocks
data_blocks_test <- data_blocks

for (i in seq_along(data_blocks)) {
  data_blocks_train[[i]] <- data_blocks_train[[i]][train_samples, ]
  data_blocks_test[[i]] <- data_blocks_test[[i]][-train_samples, ]
}

# Split corresponding metadata
metadata_train <- data.frame(metadata_NCI60[train_samples, ],
                             row.names = rownames(data_blocks_train$mrna))
colnames(metadata_train) <- c("cancerType")

metadata_test <- data.frame(metadata_NCI60[-train_samples, ],
                            row.names = rownames(data_blocks_test$mrna))
colnames(metadata_test) <- c("cancerType")

# Create train and test mae objects
data_blocks_train_mae <- simple_mae(data_blocks_train, row_format = "sample",
                                    colData = metadata_train)
data_blocks_test_mae <- simple_mae(data_blocks_test, row_format = "sample",
                                   colData = metadata_test)
```

## Run nipalsMCIA on training data

```{r computing-MCIA-on-training-data, message = FALSE, fig.show = "hide"}
MCIA_train <- nipals_multiblock(data_blocks = data_blocks_train_mae,
                                col_preproc_method = "colprofile", num_PCs = 10,
                                plots = "none", tol = 1e-9)
```

## Visualize model on training data using metadata on cancer type

The `get_metadata_colors()` function returns an assignment of a color for the
metadata columns. The `nmb_get_gs()` function returns the global scores from the
input `NipalsResult` object.

```{r visualize-training-model, fig.height = 2.5}
meta_colors <- get_metadata_colors(mcia_results = MCIA_train, color_col = 1,
                                   color_pal_params = list(option = "E"))

global_scores <- nmb_get_gs(MCIA_train)
MCIA_out <- data.frame(global_scores[, 1:2])
MCIA_out$cancerType <- nmb_get_metadata(MCIA_train)$cancerType
colnames(MCIA_out) <- c("Factor.1", "Factor.2", "cancerType")

# plot the results
ggplot(data = MCIA_out, aes(x = Factor.1, y = Factor.2, color = cancerType)) +
  geom_point(size = 3) +
  labs(title = "MCIA for NCI60 training data") +
  scale_color_manual(values = meta_colors) +
  theme_bw()
```

## Generate factor scores for test data using the MCIA_train model

We use the \verb+predict_gs+ function to generate new factor scores on the test
data set using the MCIA_train model. The new dataset in the form of an MAE object
is input using the parameter `test_data`.

```{r generate-new-scores, message = FALSE}
MCIA_test_scores <- predict_gs(mcia_results = MCIA_train,
                               test_data = data_blocks_test_mae)
```

## Visualize new scores with old

We once again plot the top two factor scores for both the training and test
datasets

```{r visualize-both-models, fig.height = 2.5}
MCIA_out_test <- data.frame(MCIA_test_scores[, 1:2])
MCIA_out_test$cancerType <-
  MultiAssayExperiment::colData(data_blocks_test_mae)$cancerType

colnames(MCIA_out_test) <- c("Factor.1", "Factor.2", "cancerType")
MCIA_out_test$set <- "test"
MCIA_out$set <- "train"
MCIA_out_full <- rbind(MCIA_out, MCIA_out_test)
rownames(MCIA_out_full) <- NULL

# plot the results
ggplot(data = MCIA_out_full,
       aes(x = Factor.1, y = Factor.2, color = cancerType, shape = set)) +
  geom_point(size = 3) +
  labs(title = "MCIA for NCI60 training and test data") +
  scale_color_manual(values = meta_colors) +
  theme_bw()
```

# Session Info

<details>
  <summary>**Session Info**</summary>
  
```{r session-info}
sessionInfo()
```

</details>

---
title: "Test Modular Functions"
output: html_document
---

```{r setup, include=FALSE}
set.seed(42)
knitr::opts_chunk$set(echo = TRUE)
```

```{r}
library(nipalsMCIA)
data("NCI60")

metadata_NCI60 <- data.frame(cancerType = unlist(metadata_NCI60))
row.names(metadata_NCI60) <- rownames(data_blocks[[1]])

# Adding metadata as part of the nipals_multiblock() function
mcia_results <- nipals_multiblock(data_blocks, preprocMethod = "colprofile",
                                  metadata = metadata_NCI60, plots = "none",
                                  num_PCs = 10, tol = 1e-12)

# Alternative method for adding metadata
mcia_results$metadata <- metadata_NCI60
```

# Test projection_plot with NO colors

```{r, fig.dim = c(4, 4), fig.align = "center"}
plot_type <- "projection_global"
orders <- c(1, 2)
projection_plot(mcia_results, projection = plot_type,
                orders = c(1, 2), color_col = NULL,
                legend_loc = "bottomleft")
```

```{r, fig.dim = c(6, 4), fig.align = "center"}
plot_type <- "projection"
projection_plot(mcia_results, projection = plot_type,
                orders = c(1, 2), color_col = NULL,
                legend_loc = "bottomleft")
```

# Test projection_plot with the default color settings (uses scales::\<function\>)

```{r}
color_col <- "cancerType"
```

```{r, fig.dim = c(4, 4), fig.align = "center"}
plot_type <- "projection_global"
orders <- c(1, 2)
projection_plot(mcia_results, projection = plot_type,
                orders = c(1, 2), color_col = "cancerType",
                legend_loc = "bottomleft")
```

```{r, fig.dim = c(6, 4), fig.align = "center"}
plot_type <- "projection"
projection_plot(mcia_results, projection = plot_type,
                orders = c(1, 2), color_col = "cancerType",
                legend_loc = "bottomleft")
```

# Test projection_plot with the user specified colors

```{r, fig.dim = c(4, 4), fig.align = "center"}
plot_type <- "projection_global"
orders <- c(1, 2)
projection_plot(mcia_results, projection = plot_type,
                orders = c(1, 2), color_col = "cancerType",
                color_pal = c("red", "green", "blue"),
                legend_loc = "bottomleft")
```

```{r, fig.dim = c(6, 4), fig.align = "center"}
plot_type <- "projection"
projection_plot(mcia_results, projection = plot_type,
                orders = c(1, 2), color_col = "cancerType",
                color_pal = c("orange", "purple", "pink"),
                legend_loc = "bottomleft")
```

# Test global_scores_eigenvalues_plot(mcia_results)

```{r, fig.dim = c(6, 4)}
global_scores_eigenvalues_plot(mcia_results)
```

# Test block_weights_heatmap

```{r, fig.dim = c(6, 4)}
block_weights_heatmap(mcia_results)
```

---
title: "Analysis of MCIA Decomposition"
author:
- Max Mattessich
- Joaquin Reyna
- Edel Aron
- Anna Konstorum
date: "Compiled: `r format(Sys.time(), '%B %d, %Y')`"
header-includes:
   - \usepackage{amsmath}
   - \usepackage{amsfonts}
output:
  BiocStyle::html_document:
    dev: "jpeg"
    df_print: kable
    fig_retina: 1
    number_sections: FALSE
    toc_depth: 3
    toc_float: TRUE
vignette: >
  %\VignetteIndexEntry{Analysis of MCIA Decomposition}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

\newcommand{\mb}[1]{\mathbf{#1}}
\newcommand{\var}{\text{var}}
\newcommand{\cov}{\text{cov}}

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Introduction

## Motivation

Multiple co-inertia analysis (MCIA) is a member of the family of joint 
dimensionality reduction (jDR) methods that extend unsupervised dimension reduction 
techniques such as Principal Components Analysis (PCA) and Non-negative Matrix 
Factorization (NMF) to datasets with multiple data blocks 
(alternatively called views) ([Cantini, 2021](https://doi.org/10.1038/s41467-020-20430-7)).

Here, we present a new implementation in R of MCIA, `nipalsMCIA`, 
that uses an extension of Non-linear Iterative Partial Least Squares (NIPALS) 
to solve the MCIA optimization problem ([Hanafi, 2011](https://doi.org/10.1016/j.chemolab.2010.05.010)). 
This implementation has several features, including speed-up over approaches that
employ the Singular Value Decomposition (SVD), several options for pre-processing 
and deflation to customize algorithm performance, methodology to perform 
out-of-sample global embedding, and analysis and visualization capabilities to 
maximize result interpretation. 

While there exist additional implementations of MCIA (e.g. [mogsa](https://www.bioconductor.org/packages/release/bioc/html/mogsa.html), 
[omicade4](https://bioconductor.org/packages/release/bioc/html/omicade4.html)), 
ours is unique in providing a pipeline that incorporates pre-processing data 
options including those present in the original development of MCIA 
(including a theoretically grounded calculation of inertia, or total variance) 
with an iterative solver that shows speed-up for larger datasets, and is 
explicitly designed for simultaneous ease of use as a tool for multi-view data 
decomposition as well as a foundation for theoretical and computational 
development of MCIA and related methodology. A manuscript detailing our 
implementation is forthcoming.

## Overview

In this vignette, we will cover the most important functions
within the `nipalsMCIA` package as well as downstream analyses that can help
interpret the MCIA decomposition using a cancer data set from
[Meng et al., 2016](https://doi.org/10.1093/bib/bbv108) that includes 21
subjects with three data blocks. The data blocks include mRNA levels
(12895 features), microRNA levels (537 features) and protein levels
(7016 features). 

The `nipals_multiblock` function performs MCIA using the NIPALS algorithm. 
`nipals_multiblock` outputs a decomposition that includes a low-dimensional 
embedding of the data in the form of global scores, and the contributions of
the data blocks (block score weights) and features (global loadings) to these
same global scores. `nipalsMCIA` provides several additional functions to
visualize, analyze, and interpret these results.

The `nipals_multiblock` function accepts as input a [MultiAssayExperiment](https://bioconductor.org/packages/release/bioc/html/MultiAssayExperiment.html)
(MAE) object. Such objects represent a modern classed-based approach to
organizing multi-omics data in which each assay can be stored as an individual
experiment alongside relevant metadata for samples and experiments. If users
have a list of data blocks with matching sample names (and optional sample-level
metadata), we provide a simple conversion function (`simple_mae.R`) to generate
an MAE object. For more sophisticated MAE object construction, please consult
the MAE documentation.

In the context of the NCI-60 data set and this vignette, we will show you the
power of MCIA to find important relationships between mRNA, microRNA and
proteins. More specifically, we will show you how to interpret the global
factor scores in [Part 1: Interpreting Global Factor Scores] and global loadings
in [Part 2: Interpreting Global Loadings].

## Installation

```{r installation-github, eval = FALSE}
# devel version

# install.packages("devtools")
devtools::install_github("Muunraker/nipalsMCIA", ref = "devel",
                         force = TRUE, build_vignettes = TRUE) # devel version
```

```{r installation-bioconductor, eval = FALSE}
# release version
if (!require("BiocManager", quietly = TRUE))
  install.packages("BiocManager")

BiocManager::install("nipalsMCIA")
```

```{r load-packages, message = FALSE}
library(ComplexHeatmap)
library(dplyr)
library(fgsea)
library(ggplot2)
library(ggpubr)
library(nipalsMCIA)
library(stringr)

# NIPALS starts with a random vector
set.seed(42)
```

## Preview of the NCI-60 dataset

The NCI-60 data set has been included with the `nipalsMCIA` package and is
easily available as shown below:

```{r intro-load-data}
# load the dataset which uses the name data_blocks
data(NCI60)

# examine the contents
data_blocks$miRNA[1:3, 1:3]
data_blocks$mrna[1:3, 1:3]
data_blocks$prot[1:3, 1:3]
```

To convert data_blocks into an MAE object we provide the `simple_mae()`
function:

```{r convert-to-mae}
data_blocks_mae <- simple_mae(data_blocks, row_format = "sample")
```

# Running and reviewing the MCIA output

We can compute the MCIA decomposition for $r$ global factors. For our example,
we take $r=10$.

```{r intro-mcia, warning = FALSE, message = FALSE}
set.seed(42)
mcia_results <- nipals_multiblock(data_blocks_mae,
                                  col_preproc_method = "colprofile",
                                  num_PCs = 10, tol = 1e-12, plots = "none")
```

The result is an NipalsResult object containing several outputs from the decomposition:

```{r intro-mcia-results}
slotNames(mcia_results)
```

We describe the first two in more detail below, and will discuss several others
in the remainder of the vignette. For additional details on the decomposition, see
([Hanafi, 2011](https://doi.org/10.1016/j.chemolab.2010.05.010), 
[Mattesich, 2022](http://hdl.handle.net/10427/CZ30Q6773)).

## Brief overview of the Global Scores Matrix ($F$)

The `global_scores` matrix is represented by $\mb F$ with dimensions
$n \times r$, where $n$ is the number of samples and $r$ is the number of
factors chosen by using the `num_PCs = r` argument. Each column $j$ of this matrix
represents the global scores for factor $j$,

$$
\mb F = \begin{pmatrix}
    | & |&  & |\\
    \mb f^{(1)} &\mb f^{(2)} & \dots & \mb f^{(r)}\\
    | & |&  & |
  \end{pmatrix} \in \mathbb{R}^{n \times r}
$$

This matrix encodes a low-dimensional representation of the data set, with the
$i$-th row representing a set of $r$-dimensional coordinates for the $i$-th
sample.

## Brief overview of the Global Loadings Matrix ($A$)

The `global_loadings` matrix is represented by $\mb A$ that is $p \times r$,
where $p$ is the number of features across all omics and $r$ is as before.
Each column $j$ of this matrix represents the global loadings for factor $j$,
i.e.

$$
\mb A = \begin{pmatrix}
    | & |&  & |\\
    \mb a^{(1)} &\mb a^{(2)} & \dots & \mb a^{(r)}\\
    | & |&  & |
  \end{pmatrix} \in \mathbb{R}^{p \times r}
$$

This matrix encodes the contribution (`loading`) of each feature to the global 
score.

The remainder of this vignette will be broken down into two sections,
[Part 1: Interpreting Global Factor Scores] and [Part 2: Interpreting Global Loadings]
where we show how to interpret $\mb F$ and $\mb A$, respectively.

# Part 1: Interpreting Global Factor Scores

## nipals_multiblock() Generates Basic Visualizations

In the introduction we showed how to calculate the MCIA decomposition using
`nipals_multiblock()` but used the parameter `plots = "none"` to avoid the
default plotting behavior of this function. By default, this function will
generate two plots which help establish an initial intuition for the MCIA
decomposition. Here we will re-run `nipals_multiblock()` with the default `plots`
parameter (`all`):

```{r part-1-mcia, warning = FALSE, message = FALSE, fig.dim = c(9, 5)}
set.seed(42)
mcia_results <- nipals_multiblock(data_blocks_mae,
                                  col_preproc_method = "colprofile",
                                  num_PCs = 10, tol = 1e-12)
```

The first plot visualizes each sample using factor 1 and 2 as a lower
dimensional representation (factor plot).

- Each sample is represented by 4 points, a center point (solid bla'ck
dot) which represents the global factor score, a mRNA factor score (square), a
miRNA factor score (circle), and a protein factor score (triangle).
- The last three omic-specific block factor scores are connected to the global
factor score. If a block factor scores is plotted far from its
corresponding global factor score, then this is an indication that the block
does not agree with/contribute to the trend found by the global decomposition.
- As an example, we can take a look at the global factor score at $(-1.1, 0.4)$.
The block factor scores are all quite near which suggests all three omics are
contributing somewhat equally. This is in contrast to the global factor score at
$(-0.3, 0.9)$ where the mRNA factor score is close, but the mRNA and miRNA are
far from their respective global factor score.

The second plot is a scree plot of normalized singular values corresponding to 
the variance explained by each factor.

## Visualizing a Factor Plot with Only Global Factor Scores

For clustering, it is useful to look at global factor scores without block
factor scores. The `projection_plot()` function can be used to generate such a
plot using `projection = "global"`.

```{r part-1-visualize-clusters, fig.dim = c(5, 5)}
projection_plot(mcia_results, projection = "global", orders = c(1, 2))
```

In addition, scores can be colored by a meaningful label such as cancer type
which is highly relevant to NCI-60. To do so, the `colData` slot of the associated
MAE object must be loaded with sample-level metadata prior to invoking
`projection_plot()`. The sample metadata is composed of row names corresponding to
the primary sample names of the MAE object, and columns contain different
metadata (e.g. age, disease status, etc). For instance, each of the 21 samples
in the NCI-60 dataset represents a cell line with one of three cancer types:
CNS, Leukemia, or Melanoma. We have provided this metadata as part of the
`data(NCI60)` dataset and we next show how it can be included in the resulting MAE
object using the `colData` parameter in `simple_mae()`:

```{r part-1-metadata}
# preview of metadata
head(metadata_NCI60)

# loading of mae with metadata
data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
                              colData = metadata_NCI60)
```

We now rerun `nipals_multiblock()` using the updated MAE object, where the
`colData` is passed to the `metadata` slot of the `NipalsResult` instance, and
then visualize the global factor scores using the `projection_plot()` function.

```{r part-1-mcia-again, warning = FALSE, message = FALSE}
# adding metadata as part of the nipals_multiblock() function
set.seed(42)
mcia_results <- nipals_multiblock(data_blocks_mae,
                                  col_preproc_method = "colprofile",
                                  plots = "none",
                                  num_PCs = 10, tol = 1e-12)
```

The `color_col` argument of `projection_plot()` can then be used to determine
which column of `metadata` is used for coloring the individual data points, in
this case `cancerType`. `color_pal` is used to assign a color palette and
requires a vector of colors (i.e. `c("blue", "red", "green")`). To help create
this vector we also provide `get_metadata_colors()`, a helper function (used 
below) that can be used with a `scales::<function>` to return an appropriate
vector of colors. Note: colors are applied by lexicographically sorting the list
of unique metadata values then assigning the first color to the first value,
second with second and so on.

```{r part-1-visualize-clusters-color-col, fig.dim = c(5, 5)}
# meta_colors = c("blue", "grey", "yellow") can use color names
# meta_colors = c("#00204DFF", "#7C7B78FF", "#FFEA46FF") can use hex codes
meta_colors <- get_metadata_colors(mcia_results, color_col = 1,
                                   color_pal_params = list(option = "E"))

projection_plot(mcia_results, projection = "global", orders = c(1, 2),
                color_col = "cancerType", color_pal = meta_colors,
                legend_loc = "bottomleft")
```

Using this plot one can observe that global factor scores for factor 1 and 2 can
separate samples into their cancer types.

## Visualizing the Clustering of Samples by Factor Scores

A heatmap can be used to cluster samples based on global scores across all factors
using `global_scores_heatmap()`. The samples can be colored by the associated metadata 
using `color_cor` + `color_pal`as shown below.

```{r part-1-global-heatmap-colored, fig.dim = c(6, 4)}
global_scores_heatmap(mcia_results = mcia_results,
                      color_col = "cancerType", color_pal = meta_colors)
```

Like the projection plot, one can observe that the clustering can differentiate
between cancer type quite effectively.

# Part 2: Interpreting Global Loadings

In addition to the global scores matrix, MCIA also calculates a global loadings
matrix $A$ that is $p\times r$.  

## Pseudoeigenvalues Representing the Contribution of Each Omic to the Global Factor Score

The global loadings matrix is calculated using a weighted sum of the block loadings.
These weights can be interpreted as the contribution of each omic to the global
factor score (i.e. *pseudoeigenvalue*), and can be visualized using the function `block_weights_heatmap()`.

```{r part-2-block-heatmap, fig.dim = c(4.5, 3)}
block_weights_heatmap(mcia_results)
```

For example, we can observe that factors 7-9 have the strongest contribution
from the protein data.

## Visualize All Feature Loadings on Two Axes

MCIA returns feature loadings for each factor that can be used to understand
important contributions. To get a sense of which omics features show high
contribution to each factor, one can plot feature loadings for any two omics
using the `vis_load_plot` function. The features that take the most extreme
values will be the strongest contributors to that factor.

```{r part-2-loadings, fig.dim = c(6, 4)}
# colors_omics = c("red", "blue", "green")
# colors_omics <- get_colors(mcia_results, color_pal = colors_omics)
colors_omics <- get_colors(mcia_results)

vis_load_plot(mcia_results, axes = c(1, 4), color_pal = colors_omics)
```

From this plot, we can deduce that individual miRNA features tend to
have the most extreme values in factors 1 and 4, thereby contributing most strongly
to the global scores for those factors.

## Scree Plot: Visualizing the Top Features per Factor

To dive into each factor users can generate a scree plot of top loading 
values using the following two steps/functions: 1) `ord_loadings()` which
calculates a list of ranked feature loadings, and 2) `vis_load_ord()` which 
takes as input ranked values from step (1) to generate a scree plot. Features
can be ranked based on ascending or descending values and/or absolute value. 
Users can also plot features from all omics together or focus on a specific
omic by using a corresponding block name (i.e. mrna). 

### Factor 1

Here, we visualize two ranked lists for Factor 1: using all omics,
or just mrna.

```{r part-2-factor-1, fig.dim = c(8, 3)}
# generate the visualizations
all_pos_1_vis <- vis_load_ord(mcia_results, omic = "all", factor = 1,
                              absolute = FALSE, descending = TRUE)
mrna_pos_1_vis <- vis_load_ord(mcia_results, omic = "mrna", factor = 1,
                               absolute = FALSE, descending = TRUE)

ggpubr::ggarrange(all_pos_1_vis, mrna_pos_1_vis)
```

The ordering itself can be easily obtained using the `ord_loadings(...)`
function. 

```{r part-2-factor-1-orderings}
# obtain the loadings as plotted above
all_pos_1 <- ord_loadings(mcia_results, omic = "all", factor = 1,
                          absolute = FALSE, descending = TRUE)
mrna_pos_1 <- ord_loadings(mcia_results, omic = "mrna", factor = 1,
                           absolute = FALSE, descending = TRUE)
```

```{r part-2-factor-1-orderings-1}
# visualize the tabular data
all_pos_1[1:3, ]
```

```{r part-2-factor-1-orderings-2}
mrna_pos_1[1:3, ]
```

The first plot shows that that miRNA has the overall top positive feature loadings for
factor 1, whereas the second plot allows us to focus on only mRNA signals,
where genes like *GNS* and *WASL* come up as top contributors.

### Factor 2

Here, we return and visualize two ranked lists for Factor 2: using all omics,
or just protein, this time ranking features by magnitude (absolute value).

```{r part-2-factor-2, fig.dim = c(8, 3)}
# define the loadings
all_abs_2 <- vis_load_ord(mcia_results, omic = "all", factor = 2,
                          absolute = TRUE, descending = TRUE)

prot_abs_2 <- vis_load_ord(mcia_results, omic = "prot", factor = 2,
                           absolute = TRUE, descending = TRUE)

ggpubr::ggarrange(all_abs_2, prot_abs_2)
```


```{r part-2-factor-2-orderings}
# obtain the loadings as plotted above
all_abs_2_vis <- ord_loadings(mcia_results, omic = "all", factor = 2,
                          absolute = TRUE, descending = TRUE)
prot_abs_2_vis <- ord_loadings(mcia_results, omic = "prot", factor = 2,
                          absolute = TRUE, descending = TRUE)
```

```{r part-2-factor-2-orderings-1}
# visualize the tabular data
all_abs_2_vis[1:3, ]
```

```{r part-2-factor-2-orderings-2}
prot_abs_2_vis[1:3, ]
```

For factor 2, we again see that across all omics, miRNA is playing the strongest
role. One important difference (relative to factor 1) is made by ranking the
values according to the absolute value. Doing so allows us to identify 9 (out
of 15) features with a strong negative loading value. The right plot shows that
the negatively values protein features have the largest absolute values, indicating 
that they may play a distinct role from the miRNA in global scores for this factor. 

### Factor 4

We return and visualize a ranked list for Factor 4 with the top 60 features.

```{r part-2-factor-4, fig.dim = c(10, 4)}
# visualization
all_4_plot <- vis_load_ord(mcia_results, omic = "all", factor = 4, 
                           absolute = FALSE, descending = TRUE, n_feat = 60)
all_4_plot
```

```{r part-2-factor-4-orderings, fig.dim = c(10, 4)}
# visualize the tabular data
all_4 <- ord_loadings(mcia_results, omic = "all", factor = 4,
                      absolute = FALSE, descending = TRUE)
```

One can observe that while miRNA dominate the top features, proteins are also 
represented.

## Pathway Analysis for the Top Factors using Data from Gene-Centric Omics Blocks

The NCI-60 data set includes gene expression data, and its corresponding global
loading matrix is a gene by factor matrix. We can learn more about the pathways
over-represented in a given factor by running the global loadings for the factor 
through a gene set enrichment analysis (GSEA). Here, we look at factors 1 and 3.
We run `gsea_report()`, which reports on the p-value of the most significant 
pathway as well as the total number of significant pathways for each factor. 

### Gather Data and Generate the Report

```{r part-2-pathways, echo = TRUE, message = FALSE, warning = FALSE, results = "hide"}
# extract mRNA global loadings
mrna_gfscores <- nmb_get_gl(mcia_results)
mrna_rows <- str_detect(row.names(mrna_gfscores), "_mrna")
mrna_gfscores <- mrna_gfscores[mrna_rows, ]

# rename rows to contain HUGO based gene symbols
row.names(mrna_gfscores) <- str_remove(rownames(mrna_gfscores), "_[0-9]*_.*")

# load pathway data
path.database <- "https://data.broadinstitute.org/gsea-msigdb/msigdb/release/6.2/c2.cp.reactome.v6.2.symbols.gmt"
pathways <- fgsea::gmtPathways(gmt.file = path.database)

# generate the GSEA report
geneset_report <- gsea_report(metagenes = mrna_gfscores, path.database,
                              factors = seq(1, 3), pval.thr = 0.05, nproc = 2)
```

### Investigating the GSEA Summary Table

The report comes in the form of a list where the first element is a data frame
with summary level of the GSEA analysis per factor. Ideally, each factor is
capturing a select number of pathways with a high significance. From this
report (below) we can see that the most significant pathway is associated with
Factor 3 and that there is a large variation in the number of total (significant)
pathways ranging from 7 (Factor 8) to 143 (Factor 4).

```{r include = FALSE}
# # Apply scientific notation to min_pval
geneset_report[[1]]$min_pval <- sprintf("%.2e", geneset_report[[1]]$min_pval)
```

```{r part-2-genesets}
geneset_report[[1]]
```

As just mentioned, Factor 3 contains the most enriched gene set so we can
re-run GSEA for this factor in order to get a full list of enrichment scores
across all gene sets:

```{r part-2-gsea-for-factor-3, echo = TRUE, message = FALSE, warning = FALSE, results = "hide"}
# re-running GSEA
factor3_paths <- fgseaMultilevel(pathways, stats = mrna_gfscores[, 3],
                                 nPermSimple = 10000, minSize = 15,
                                 nproc = 2, maxSize = 500)
```

```{r include=FALSE}
# Order by adjusted p value
factor3_paths <- factor3_paths[order(factor3_paths$padj), ]

# Apply scientific notation to padj
to_sci <- function(x) return(sprintf("%.2e", x))
factor3_paths$padj <- vapply(factor3_paths$padj, to_sci, character(1))

# Clean up pathway name
clean_pathway_name <- function(x) {
    replaced_value <- str_replace_all(x , "_", " ")
    if (nchar(replaced_value) > 49) {
        substring_value <- substr(replaced_value, 1, 50) 
        return(paste0(substring_value, "..."))
    }
    else {
        return(replaced_value)
    }
}

factor3_paths[, "pathway"] <- apply(X = factor3_paths[, "pathway"],
                                    MARGIN = 1, FUN = clean_pathway_name)
```

```{r part-2-gsea-for-factor-3-viz}
head(factor3_paths[, c("pathway", "padj")])
```

We observe that the most significant gene set is the REACTOME_CELL_CYCLE gene set. 
Below we can take a look at the list of genes from REACTOME_CELL_CYCLE that
likely contribute to factor 3. This analysis can be repeated as necessary to
make sense of other gene based factor loadings. 

```{r part-2-significant}
# extracting REACTOME_CELL_CYCLE, the most significant gene set
sig_path3 <- factor3_paths[min(factor3_paths$padj) == factor3_paths$padj, ][1, ]
sig_path3$leadingEdge[[1]][1:10]
```

# Session Info

<details>
  <summary>**Session Info**</summary>
  
```{r session-info}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nipals_multiblock.R
\name{nipals_multiblock}
\alias{nipals_multiblock}
\title{Main NIPALS computation loop}
\usage{
nipals_multiblock(
  data_blocks_mae,
  col_preproc_method = "colprofile",
  block_preproc_method = "unit_var",
  num_PCs = 10,
  tol = 1e-09,
  max_iter = 1000,
  color_col = NULL,
  deflationMethod = "block",
  plots = "all",
  harmonize = TRUE
)
}
\arguments{
\item{data_blocks_mae}{a MultiAssayExperiment class
object (with sample metadata as a dataframe in the colData attribute).}

\item{col_preproc_method}{an option for the desired column-level data
pre-processing, either:
\itemize{
\item `colprofile` applies column-centering, row and column weighting by
contribution to variance.
\item `standardized` centers each column and divides by its standard
deviation.
\item `centered_only` ONLY centers data
}}

\item{block_preproc_method}{an option for the desired block-level data
pre-processing, either:\itemize{
\item `unit_var` FOR CENTERED MATRICES ONLY - divides each block by the
square root of its variance
\item `num_cols` divides each block by the number of variables in the block.
\item `largest_sv` divides each block by its largest singular value.
\item `none` performs no preprocessing
}}

\item{num_PCs}{the maximum order of scores/loadings}

\item{tol}{a number for the tolerance on the stopping criterion for NIPALS}

\item{max_iter}{a number for the maximum number of times NIPALS should
iterate}

\item{color_col}{Optional argument with the column name of the `metadata`
data frame used to define plotting colors}

\item{deflationMethod}{an option for the desired deflation method, either:
\itemize{
\item `block` deflation via block loadings (for MCIA, default)
\item `global` deflation via global scores (for CPCA)
}}

\item{plots}{an option to display various plots of results: \itemize{
\item `all` displays plots of block scores, global scores, and eigenvalue
scree plot
\item `global` displays only global score projections and eigenvalue scree
plot
\item `none` does not display plots
}}

\item{harmonize}{boolean whether or not samples should be checked for duplicates 
and re-ordered so that each row corresponds to the same sample across datasets.
Set to FALSE to greatly reduce computation time on many samples (default = TRUE).}
}
\value{
a `nipalsResult` object with the following fields: \itemize{
\item `global_scores` a matrix containing global scores as columns
(NOT normalized to unit variance)
\item `global_loadings` a matrix containing global loadings as columns
\item `global_score_weights` a matrix of weights to express global scores as
a combination of block scores. Has dimensions "num_Blocks" by "num_PCs"
\item `eigvals` a matrix containing the eigenvalue for each computed global
score.
\item `block scores` a list of matrices, each contains the scores for one
block
\item `block loadings` a list of matrices, each contains the loadings for
one block (w/ unit length)
\item `block score weights` a matrix containing weights for each block score
of each order used to construct the global scores.
\item `col_preproc_method` the column preprocessing method used on the data.
\item `block_variances` a list of variances of each block AFTER
NORMALIZATION OPTION APPLIED
\item `metadata` the metadata dataframe supplied with the `metadata`
argument. Note: overrides metadata present in any MAE class object.
}
}
\description{
Applies the full adjusted NIPALS algorithm to generate block
and global scores/loadings with the desired deflation method.
}
\details{
Follows the NIPALS algorithm as described by Hanafi et. al. (2010).
For each order of scores/loadings, the vectors are computed via the
`nipals_iter` function, then used to deflate the data matrix according to
the desired deflation method.
This process is repeated up to the desired maximum order of scores/loadings.
}
\examples{
   data(NCI60)
   data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                                 colData=metadata_NCI60)
   NIPALS_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                       tol = 1e-12, max_iter = 1000,
                                       col_preproc_method = "colprofile",
                                       deflationMethod = "block")
   MCIA_results <- nipals_multiblock(data_blocks_mae, num_PCs = 4)
   CPCA_results <- nipals_multiblock(data_blocks_mae, num_PCs = 4,
   deflationMethod = 'global')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_tv.R
\name{get_tv}
\alias{get_tv}
\title{Computes the total variance of a multi-omics dataset}
\usage{
get_tv(ds)
}
\arguments{
\item{ds}{a list of multi-omics dataframes/matrices in "sample x variable"
format}
}
\value{
the total variance of the dataset (i.e. sum of block variances)
}
\description{
Computes the total variances of all data blocks in a
multi-omics dataset, intended for datasets that do not use `CCpreproc`
}
\examples{
data(NCI60)
tot_var <- get_tv(data_blocks)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea_report.R
\name{gsea_report}
\alias{gsea_report}
\title{Perform biological annotation-based comparison}
\usage{
gsea_report(
  metagenes,
  path.database,
  factors = NULL,
  pval.thr = 0.05,
  nproc = 4
)
}
\arguments{
\item{metagenes}{Vector of gene scores where the row names are HUGO symbols}

\item{path.database}{path to a GMT annotation file}

\item{factors}{vector of factors which should be analyzed}

\item{pval.thr}{p-value threshold (default to 0.05)}

\item{nproc}{number of processors to utilize}
}
\value{
data frame with the most significant p-value number of significant
pathways

the selectivity scores across the given factors
}
\description{
Runs fgsea for the input gene vector
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{NipalsResult-class}
\alias{NipalsResult-class}
\alias{NipalsResult}
\title{An S4 class to contain results computed with `nipals_multiblock()`}
\value{
A NipalsResult object.
}
\description{
An S4 class to contain results computed with `nipals_multiblock()`
}
\section{Slots}{

\describe{
\item{\code{global_scores}}{A matrix containing global scores as columns.}

\item{\code{global_loadings}}{A matrix containing global loadings as columns.}

\item{\code{block_score_weights}}{A matrix containing block weights as columns.}

\item{\code{block_scores}}{A list of matrices. Each matrix contains the scores
as columns for a given block.}

\item{\code{block_loadings}}{A list of matrices. Each matrix contains the
loadings as columns for a given block.}

\item{\code{eigvals}}{A list of singular values of the data matrix at each
deflation step.}

\item{\code{col_preproc_method}}{character for the column-level preprocessing
method used. See `col_preproc()`.}

\item{\code{block_preproc_method}}{character for the block-level
preprocessing method used. See `block_preproc()`.}

\item{\code{block_variances}}{A list of variances for each block.}

\item{\code{metadata}}{A data frame of metadata originally passed into
`nipals_multiblock()`.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_scores_heatmap.R
\name{global_scores_heatmap}
\alias{global_scores_heatmap}
\title{Plotting a heatmap of global factors scores (sample v. factors)}
\usage{
global_scores_heatmap(
  mcia_results,
  color_col = NULL,
  color_pal = scales::viridis_pal,
  color_pal_params = list(option = "D")
)
}
\arguments{
\item{mcia_results}{the mcia object matrix after running MCIA, must also
contain metadata with columns corresponding to color_col}

\item{color_col}{an integer or string specifying the column that will be
used for color_col}

\item{color_pal}{a list of colors or function which returns a list of colors}

\item{color_pal_params}{a list of parameters for the color function}
}
\value{
ComplexHeatmap object
}
\description{
Plots a heatmap of MCIA global scores
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmb_accessors.R
\name{nmb_get_eigs}
\alias{nmb_get_eigs}
\title{Accessor function for eigenvalues}
\usage{
nmb_get_eigs(nmb_object)
}
\arguments{
\item{nmb_object}{A `NipalsResult` object.}
}
\value{
a matrix containing the eigenvalues for all global scores.
}
\description{
Retrieves the eigenvalues
from a `NipalsResult` object, typically output from `nipals_multiblock()`.
}
\examples{
data("NCI60")
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
nipals_eigvals <- nmb_get_eigs(mcia_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_metadata_colors.R
\name{get_metadata_colors}
\alias{get_metadata_colors}
\title{Assigning colors to different values of a metadata column}
\usage{
get_metadata_colors(
  mcia_results,
  color_col,
  color_pal = scales::viridis_pal,
  color_pal_params = list()
)
}
\arguments{
\item{mcia_results}{object returned from nipals_multiblock() function}

\item{color_col}{an integer or string specifying the column that will be
used for color_col}

\item{color_pal}{a function which returns color palettes (e.g. scales)}

\item{color_pal_params}{list of parameters for the corresponding function}
}
\value{
List of metadata columns with assigned colors
}
\description{
Creates a list of metadata columns and associated colors
for plotting. The default palette was chosen to be color-blindness friendly.
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                               colData=metadata_NCI60)
mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                  plots = "none", tol = 1e-12)
colors_omics <- get_metadata_colors(mcia_results, "cancerType",
                                    color_pal_params = list(option = "E"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmb_accessors.R
\name{nmb_get_bl}
\alias{nmb_get_bl}
\title{Accessor function for block loadings}
\usage{
nmb_get_bl(nmb_object)
}
\arguments{
\item{nmb_object}{A `NipalsResult` object.}
}
\value{
a list of matrices containing block loadings.
}
\description{
Retrieves the block loadings as a list of matrices
from a `NipalsResult` object, typically output from `nipals_multiblock()`.
}
\examples{
data("NCI60")
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
block_loadings<- nmb_get_bl(mcia_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cc_preproc.R
\name{cc_preproc}
\alias{cc_preproc}
\title{Centered Column Profile Pre-processing}
\usage{
cc_preproc(df)
}
\arguments{
\item{df}{the data frame to apply pre-processing to, in "sample" x
"variable" format}
}
\value{
the processed data frame
}
\description{
Converts data blocks into centered column profiles where each
block has unit variance. Mimics the pre-processing in the Omicade4 package
(Meng et al. 2014)
}
\details{
Performs the following steps on a given data frame: \itemize{
\item Offsets data to make whole matrix non-negative
\item Divides each column by its sum
\item Subtracts (row sum/total sum) from each row
\item Multiplies each column by sqrt(column sum/total sum)
\item Divides the whole data frame by its total variance (the sqrt of the
sum of singular values)
}
}
\examples{
df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
preprocessed_dataframe <- cc_preproc(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simple_mae.R
\name{simple_mae}
\alias{simple_mae}
\title{Create an MAE object from a list of data matrices and column data}
\usage{
simple_mae(matrix_list, row_format = "feature", colData_input = NULL)
}
\arguments{
\item{matrix_list}{named list of data matrices}

\item{row_format}{for lists of data frames, indicates whether rows of
datasets denote `feature` (default) or `sample`.}

\item{colData_input}{a data frame containing sample metadata; sample names
in the rownames should correspond to samples names in `matrix_list`}
}
\value{
List of harmonized data matrices for input into nipals_multiblock()
}
\description{
Create an MAE object from a set of data matrices and column data.
}
\details{
Requires that sample names match across experiments and are identical
to primary names, will only convert data matrices to SummarizedExperiment class.
If the data is more complex, please follow the guidelines
for creating an MAE object outlined in `help(MultiAssayExperiment)`
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks, row_format = "sample",
                              colData = metadata_NCI60)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projection_plot.R
\name{projection_plot}
\alias{projection_plot}
\title{projection_plot}
\usage{
projection_plot(
  mcia_results,
  projection,
  orders = c(1, 2),
  block_name = NULL,
  color_col = NULL,
  color_pal = scales::viridis_pal,
  color_pal_params = list(option = "E"),
  legend_loc = "bottomleft",
  color_override = NULL,
  cex = 0.5
)
}
\arguments{
\item{mcia_results}{MCIA results object returned from `nipals_multiblock`}

\item{projection}{of plot, with the following options
\itemize{
\item `all` - scatter plot of two orders of global and block
              scores (aka factors).
\item `global` - scatter plot of two orders of global scores
                 only (aka factors).
\item `block` - scatter plot of two orders of block scores
                only (aka factors) for given block.
}}

\item{orders}{Option to select orders of factors to plot against each other
(for projection plots)}

\item{block_name}{Name of the block to be plotted (if `projection = block` argument used).}

\item{color_col}{an integer or string specifying the column that will be
used for color_col}

\item{color_pal}{a list of colors or function which returns a list of colors}

\item{color_pal_params}{a list of parameters for the color function}

\item{legend_loc}{Option for legend location, or "none" for no legend.}

\item{color_override}{Option to override colors when necessary, helpful for
projection = "global" or "block"}

\item{cex}{Resizing parameter for drawing the points}
}
\value{
Displays the desired plots
}
\description{
Function to generate a projection plot of MCIA results.
}
\details{
Plotting function for a projection plot.
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                  plots = "none", tol = 1e-12)
projection_plot(mcia_results, projection = "all", orders = c(1,2),
                color_col = "cancerType", legend_loc = "bottomright")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_scores_eigenvalues_plot.R
\name{global_scores_eigenvalues_plot}
\alias{global_scores_eigenvalues_plot}
\title{global_scores_eigenvalues_plot}
\usage{
global_scores_eigenvalues_plot(mcia_results)
}
\arguments{
\item{mcia_results}{MCIA results object returned from `nipals_multiblock`}
}
\value{
Displays the contribution plot using eigenvalues
}
\description{
Function to plot eigenvalues of scores up to num_PCs
}
\details{
Plotting function for eigenvalues of scores up to num_PCs
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                             colData=metadata_NCI60)
mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                 plots = "none", tol=1e-12)
global_scores_eigenvalues_plot(mcia_results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deflate_block_gs.R
\name{deflate_block_gs}
\alias{deflate_block_gs}
\title{Deflation via global scores}
\usage{
deflate_block_gs(df, gs)
}
\arguments{
\item{df}{a data frame in "sample" x "variable" format}

\item{gs}{a global scores vector in sample space}
}
\value{
the deflated data frame
}
\description{
Removes data from a data frame in the direction of a given
global scores vector.
}
\details{
Subtracts the component of each column in the direction of a given
global scores vector to yield a `deflated' data matrix.
}
\examples{
df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
global_score <- rbinom(5, 1, prob = 0.3)
deflated_data <- deflate_block_gs(df, global_score)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmb_accessors.R
\name{nmb_get_gl}
\alias{nmb_get_gl}
\title{Accessor function for global loadings}
\usage{
nmb_get_gl(nmb_object)
}
\arguments{
\item{nmb_object}{A `NipalsResult` object.}
}
\value{
a matrix containing global loadings.
}
\description{
Retrieves the global loadings as a matrix from a `NipalsResult`
object, typically output from `nipals_multiblock()`.
}
\examples{
data("NCI60")
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                   colData=metadata_NCI60)
mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
global_loadings <- nmb_get_gl(mcia_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmb_accessors.R
\name{nmb_get_metadata}
\alias{nmb_get_metadata}
\title{Accessor function for metadata}
\usage{
nmb_get_metadata(nmb_object)
}
\arguments{
\item{nmb_object}{A `NipalsResult` object.}
}
\value{
a dataframe containing metadata associated with the `NipalsResult` object.
}
\description{
Retrieves the metadata
from a `NipalsResult` object, typically output from `nipals_multiblock()`.
}
\examples{
data("NCI60")
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
nipals_metadata <- nmb_get_metadata(mcia_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nipals_iter.R
\name{nipals_iter}
\alias{nipals_iter}
\title{NIPALS Iteration}
\usage{
nipals_iter(ds, tol = 1e-12, maxIter = 1000)
}
\arguments{
\item{ds}{a list of data matrices, each in "sample" x "variable" format}

\item{tol}{a number for the tolerance on the stopping criterion for NIPALS}

\item{maxIter}{a number for the maximum number of times NIPALS should iterate}
}
\value{
a list containing the global/block scores, loadings and weights for
a given order
}
\description{
Applies one iteration stage/loop of the NIPALS algorithm.
}
\details{
Follows the NIPALS algorithm as described by Hanafi et. al. (2010).
Starts with a random vector in sample space and repeatedly projects it onto
the variable vectors and block scores to generate block and global
loadings/scores/weights. The loop stops when either the stopping criterion
is low enough, or the maximum number of iterations is reached. Intended as a
utility function for `nipals_multiblock` to be used between deflation steps.
}
\examples{
data(NCI60)
data_blocks <- lapply(data_blocks, as.matrix)
nipals_results <- nipals_iter(data_blocks, tol = 1e-7, maxIter = 1000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_NCI60.R
\name{metadata_NCI60}
\alias{metadata_NCI60}
\title{NCI-60 Multi-Omics Metadata}
\source{
Meng et. al, 2016 supplementary materials {https://doi.org/10.1093/bib/bbv108}
}
\value{
List with 21 elements
}
\description{
Metadata for the included multi-omics dataset, denoting
the cancer type associated with each of the 21 cell lines.
}
\references{
\url{https://github.com/aedin/NCI60Example}
}
\keyword{data}
\keyword{multi-omics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/col_preproc.R
\name{col_preproc}
\alias{col_preproc}
\title{Centered Column Profile Pre-processing}
\usage{
col_preproc(df, col_preproc_method)
}
\arguments{
\item{df}{the data frame to apply pre-processing to, in "sample" x
"variable" format}

\item{col_preproc_method}{denotes the type of column-centered preprocessing.
Options are: \itemize{
\item `colprofile` Performs the following steps on a given data frame:
\itemize{
\item Offsets data to make whole matrix non-negative
\item Divides each column by its sum
\item Subtracts (row sum/total sum) from each row
\item Multiplies each column by sqrt(column sum/total sum)
}
\item `standardized` centers each column and divides by its standard
deviation.
\item `centered_only` ONLY centers data
}}
}
\value{
the processed data frame
}
\description{
Converts data blocks into centered column profiles where each
block has unit variance. Mimics the pre-processing in the Omicade4 package
(Meng et al. 2014)
}
\details{
Performs preprocessing on a sample/variable (row/column) level
according to the parameter given.
}
\examples{
df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
preprocessed_dataframe <- col_preproc(df, col_preproc_method = 'colprofile')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_from_mae.R
\name{extract_from_mae}
\alias{extract_from_mae}
\title{Extract a list of harmonized data matrices from an MAE object}
\usage{
extract_from_mae(MAE_object, subset_data = "all", harmonize = TRUE)
}
\arguments{
\item{MAE_object}{an MAE object containing experiment data for extraction
colData field optional
experiments should either be SummarizedExperiment, SingleCellExperiment, or
RangedSummarizedExperiment classes}

\item{subset_data}{\itemize{
\item `all` use all experiments in MAE object
\item `c(omic1,omic2,...)` list of omics from names(MAE_object)
}}

\item{harmonize}{A boolean whether samples should be checked for duplicates \itemize{
\item `TRUE` (default) merges duplicate samples via the `MultiAssayExperiment::mergeReplicates` function
\item `FALSE` skips sample duplicate check - USE THIS FOR LARGE-SAMPLE DATASETS.
}}
}
\value{
List of harmonized data matrices for input into `nipals_multiblock()`
}
\description{
Extract a list of harmonized data matrices for
input into nipals_multiblock() from an MAE object
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
NCI60_input = extract_from_mae(data_blocks_mae,subset='all')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_gs.R
\name{predict_gs}
\alias{predict_gs}
\title{Prediction of new global scores based on block loadings and weights}
\usage{
predict_gs(mcia_results, test_data)
}
\arguments{
\item{mcia_results}{an mcia object output by nipals_multiblock()
containing block scores, weights, and pre-processing identifier.}

\item{test_data}{an MAE object with the same block types and features as the
training dataset. Feature and omic order must match `bl`.}
}
\value{
a matrix of predicted global scores for the training data
}
\description{
Uses previously-computed block scores and weights to compute
a global score for new data. Only validated for MCIA results, as CPCA loadings
aren't compatible with un-deflated data.
}
\details{
Projects the new observations onto each block loadings vector, then
weights the projection according to the corresponding block weights.
}
\examples{
   data(NCI60)
   data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                                 colData=metadata_NCI60)
   mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 2)
   new_data <- data_blocks_mae # should update with a truly new dataset
   preds <- predict_gs(mcia_results, new_data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_load_plot.R
\name{vis_load_plot}
\alias{vis_load_plot}
\title{Visualize all loadings on two factor axes}
\usage{
vis_load_plot(
  mcia_results,
  axes = c(1, 2),
  color_pal = scales::viridis_pal,
  color_pal_params = list()
)
}
\arguments{
\item{mcia_results}{object returned from nipals_multiblock() function}

\item{axes}{list of two numbers associated with two factors to visualize}

\item{color_pal}{a list of colors or function which returns a list of colors}

\item{color_pal_params}{a list of parameters for the color function}
}
\value{
Plot of MCIA feature loadings for chosen axes
}
\description{
Visualize all loadings recovered from
nipalsMCIA() output loadings matrix ranked using across two factor axes
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                  plots = "none", tol = 1e-12)
vis_load_plot(mcia_results, axes = c(1, 4))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_load_ord.R
\name{vis_load_ord}
\alias{vis_load_ord}
\title{Visualize ranked loadings}
\usage{
vis_load_ord(
  mcia_results,
  omic,
  factor = 1,
  n_feat = 15,
  absolute = TRUE,
  descending = TRUE,
  color_pal = scales::viridis_pal,
  color_pal_params = list()
)
}
\arguments{
\item{mcia_results}{object returned from nipals_multiblock() function}

\item{omic}{name of the given omic dataset}

\item{factor}{choose a factor (numeric value from 1 to number of factors
in mcia_results)}

\item{n_feat}{number of features to visualize}

\item{absolute}{whether to rank by absolute value}

\item{descending}{whether to rank in descending or ascending order}

\item{color_pal}{a list of colors or function which returns a list of colors}

\item{color_pal_params}{a list of parameters for the color function}
}
\value{
Plot in features for a factor by rank
}
\description{
Visualize a scree plot of loadings recovered from
nipalsMCIA() output loadings matrix ranked using the
ord_loadings() functions
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                  plots = "none", tol = 1e-12)
vis_load_ord(mcia_results, omic="mrna")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmb_accessors.R
\name{nmb_get_gs}
\alias{nmb_get_gs}
\title{Accessor function for global scores}
\usage{
nmb_get_gs(nmb_object)
}
\arguments{
\item{nmb_object}{A `NipalsResult` object.}
}
\value{
a matrix containing global scores.
}
\description{
Retrieves the global scores as a matrix from a `NipalsResult`
object, typically output from `nipals_multiblock()`.
}
\examples{
data("NCI60")
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                               colData=metadata_NCI60)
mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
global_scores <- nmb_get_gs(mcia_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_colors.R
\name{get_colors}
\alias{get_colors}
\title{Assigning colors to different omics}
\usage{
get_colors(
  mcia_results,
  color_pal = scales::viridis_pal,
  color_pal_params = list()
)
}
\arguments{
\item{mcia_results}{object returned from nipals_multiblock() function}

\item{color_pal}{a function which returns color palettes (e.g. scales)}

\item{color_pal_params}{list of parameters for the corresponding function}
}
\value{
List of omics with assigned colors
}
\description{
Creates a list of omics and associated colors for plotting. The
default palette was chosen to be color-blindness friendly.
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                 plots = "none", tol = 1e-12)
colors_omics <- get_colors(mcia_results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nipalsMCIA-package.R
\docType{package}
\name{nipalsMCIA-package}
\alias{nipalsMCIA}
\alias{nipalsMCIA-package}
\title{nipalsMCIA: Multiple Co-Inertia Analysis via the NIPALS Method}
\description{
Computes Multiple Co-Inertia Analysis (MCIA), a dimensionality reduction (jDR) algorithm, for a multi-block dataset using a modification to the Nonlinear Iterative Partial Least Squares method (NIPALS) proposed in (Hanafi et. al, 2010). Allows multiple options for row- and table-level preprocessing, and speeds up computation of variance explained. Vignettes detail application to bulk- and single cell- multi-omics studies.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Muunraker/nipalsMCIA}
  \item Report bugs at \url{https://github.com/Muunraker/nipalsMCIA/issues}
}

}
\author{
\strong{Maintainer}: Maximilian Mattessich \email{maximilian.mattessich@northwestern.edu} (\href{https://orcid.org/0000-0002-1233-1240}{ORCID})

Authors:
\itemize{
  \item Joaquin Reyna \email{joreyna@live.com} (\href{https://orcid.org/0000-0002-8468-2840}{ORCID})
  \item Edel Aron \email{edel.aron@yale.edu} (\href{https://orcid.org/0000-0002-8683-4772}{ORCID})
  \item Ferhat Ay \email{ferhatay@lji.org} (\href{https://orcid.org/0000-0002-0708-6914}{ORCID})
  \item Steven Kleinstein \email{steven.kleinstein@yale.edu} (\href{https://orcid.org/0000-0003-4957-1544}{ORCID})
  \item Anna Konstorum \email{konstorum.anna@gmail.com} (\href{https://orcid.org/0000-0003-4024-2058}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/block_weights_heatmap.R
\name{block_weights_heatmap}
\alias{block_weights_heatmap}
\title{block_weights_heatmap}
\usage{
block_weights_heatmap(mcia_results)
}
\arguments{
\item{mcia_results}{MCIA results object returned from `nipals_multiblock`}
}
\value{
heatmap object containing the block weights as a heatmap
}
\description{
Function to plot heatmap of block score weights
}
\details{
Plotting function for heatmap of block score weights
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                  plots = "none", tol = 1e-12)
block_weights_heatmap(mcia_results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/block_preproc.R
\name{block_preproc}
\alias{block_preproc}
\title{Block-level preprocessing}
\usage{
block_preproc(df, block_preproc_method)
}
\arguments{
\item{df}{dataset to preprocess (must be in data matrix form)}

\item{block_preproc_method}{method which is used to normalize blocks, with
options: \itemize{
\item `unit_var` FOR CENTERED MATRICES ONLY - divides each block by the
square root of its variance
\item `num_cols` divides each block by the number of variables in the block.
\item `largest_sv` divides each block by its largest singular value.
\item `none` performs no preprocessing
}}
}
\value{
the preprocessed dataset
}
\description{
A function that normalizes an input dataset (data block)
according to a variety of options.
Intended to be used after column/row-level normalization.
}
\examples{
df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
preprocessed_dataframe <- block_preproc(df,"unit_var")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deflate_block_bl.R
\name{deflate_block_bl}
\alias{deflate_block_bl}
\title{Deflation via block loadings}
\usage{
deflate_block_bl(df, bl)
}
\arguments{
\item{df}{a data frame in "sample" x "variable" format}

\item{bl}{a block loadings vector in variable space}
}
\value{
the deflated data frame
}
\description{
Removes data from a data frame in the direction of a given
block loadings vector.
}
\details{
Subtracts the component of each row in the direction of a given
block loadings vector to yield a `deflated' data matrix.
}
\examples{
df <- matrix(rbinom(15, 1, prob = 0.3), ncol = 3)
block_loading <- rbinom(3, 1, prob = 0.3)
deflated_data <- deflate_block_bl(df, block_loading)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmb_accessors.R
\name{nmb_get_bs_weights}
\alias{nmb_get_bs_weights}
\title{Accessor function for block score weights}
\usage{
nmb_get_bs_weights(nmb_object)
}
\arguments{
\item{nmb_object}{A `NipalsResult` object.}
}
\value{
a matrix containing the block score weights.
}
\description{
Retrieves the block score weights
from a `NipalsResult` object, typically output from `nipals_multiblock()`.
}
\examples{
data("NCI60")
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                             colData=metadata_NCI60)
mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
block_score_weights <- nmb_get_bs_weights(mcia_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmb_accessors.R
\name{nmb_get_bs}
\alias{nmb_get_bs}
\title{Accessor function for block scores}
\usage{
nmb_get_bs(nmb_object)
}
\arguments{
\item{nmb_object}{A `NipalsResult` object.}
}
\value{
a list of matrices containing block scores.
}
\description{
Retrieves the block scores as a list of matrices
from a `NipalsResult` object, typically output from `nipals_multiblock()`.
}
\examples{
data("NCI60")
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_out <- nipals_multiblock(data_blocks_mae, num_PCs = 10)
block_scores <- nmb_get_bs(mcia_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ord_loadings.R
\name{ord_loadings}
\alias{ord_loadings}
\title{Ranked global loadings dataframe}
\usage{
ord_loadings(
  mcia_results,
  omic = "all",
  factor = 1,
  absolute = FALSE,
  descending = TRUE
)
}
\arguments{
\item{mcia_results}{object returned from nipals_multiblock() function}

\item{omic}{choose an omic to rank, or choose 'all' for all,
((omic = "all", omic = "miRNA", etc.))}

\item{factor}{choose a factor (numeric value from 1 to number of factors
in mcia_results)}

\item{absolute}{whether to rank by absolute value}

\item{descending}{whether to rank in descending or ascending order}
}
\value{
ranked dataframe
}
\description{
Creates a dataframe with ranked loadings for a given factor
}
\examples{
data(NCI60)
data_blocks_mae <- simple_mae(data_blocks,row_format="sample",
                              colData=metadata_NCI60)
mcia_results <- nipals_multiblock(data_blocks_mae, num_PCs = 10,
                                  plots = "none", tol = 1e-12)
all_pos_1 <- ord_loadings(mcia_results = mcia_results, omic = "all",
                          absolute = FALSE, descending = TRUE, factor = 1)
}
